//
//  AppDelegate.m
//  TeKtoTut
//
//  Created by Admin on 01.12.12.
//
//

#import "AppDelegate.h"

#import "NavigationController.h"
#import "LocationManager.h"
#import "MenuContainerViewController.h"
#import "LoginViewController.h"
#import "APIRequest.h"

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    self.window = [[UIWindow alloc]initWithFrame:[UIScreen mainScreen].bounds];
    [self createMenuController];
    [self.window makeKeyAndVisible];
    
    [application registerForRemoteNotificationTypes: UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound | UIRemoteNotificationTypeAlert];
    
    NSDictionary *remoteNotificationDictionary = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
    if(remoteNotificationDictionary) {
        [self handleRemoteNotification:remoteNotificationDictionary];
    }
    
    [self addNotificationsObserver];
    return YES;
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    [LocationManager startUpdatingLocation];
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    [LocationManager stopUpdatingLocation];
}

#pragma mark - Remote notifications methods
- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    NSLog(@"TOKEN %@",deviceToken);
    NSString *stringToken = [NSString stringWithFormat:@"%@", deviceToken];
    stringToken = [stringToken stringByReplacingOccurrencesOfString:@" " withString:@""];
    stringToken = [stringToken stringByReplacingOccurrencesOfString:@"<" withString:@""];
    stringToken = [stringToken stringByReplacingOccurrencesOfString:@">" withString:@""];
    NSLog(@"STRING TOKEN %@",stringToken);
    if(isUserLogin()) {
        [APIRequest updatePushNotificationToken:stringToken];
    }
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    NSLog(@"TOKEN ERROR %@",error);
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo
{
    if(application.applicationState == UIApplicationStateInactive) {
        [self handleRemoteNotification:userInfo];
    }
}

#pragma mark - singleton method
+ (AppDelegate *)delegate
{
    return [UIApplication sharedApplication].delegate;
}

#pragma mark - working methods
- (void)createMenuController
{
    MenuContainerViewController *controller = [MenuContainerViewController new];
    self.window.rootViewController = controller;
    self.menuController = controller;
}

- (void)handleRemoteNotification:(NSDictionary *)notificationInfo
{
    NSLog(@"HANDLE NOTIFICATION");
}


#pragma mark - notifications methods
- (void)addNotificationsObserver
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(needToLogin:) name:kNeedLoginNotification object:nil];
}

- (void)needToLogin:(id)notification
{
    LoginViewController *controller = [LoginViewController new];
    controller.delegate = (id<LoginViewControllerDelegate>)self;
    NavigationController *navController = [[NavigationController alloc]initWithRootViewController:controller];
    [self.window.rootViewController presentModalViewController:navController animated:YES];
}

#pragma mark - loginViewControllerDelegate
- (void)didLoginWithServerID:(NSInteger)serverID serverToken:(NSString *)serverToken
{
    [self.window.rootViewController dismissModalViewControllerAnimated:YES];
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:serverToken forKey:kServerAccessTokenKey];
    [defaults setObject:[NSNumber numberWithInteger:serverID] forKey:kServerUserID];
    [defaults synchronize];
}

- (void)didCancelLogin
{
    [self.window.rootViewController dismissModalViewControllerAnimated:YES];
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:nil forKey:kServerAccessTokenKey];
    [defaults setObject:nil forKey:kServerUserID];
    [defaults synchronize];
}
@end
