//
//  UIImageView+RemoteImage.m
//  TeKtoTut
//
//  Created by Admin on 02.12.12.
//
//

#import "UIImageView+RemoteImage.h"

#import "ActivityView.h"
#import "ImageDownloader.h"
#import "NSString+MD5.h"

@implementation UIImageView (RemoteImage)

- (void)startLoadImageFromNetwork
{
    ActivityView *actView = [[ActivityView alloc]init];
    [self addSubview:actView];
    actView.center = CGPointMake(CGRectGetMidX(self.bounds), CGRectGetMidY(self.bounds));
    [actView startAnimating];
}

- (void)removeLoadingView
{
    for(UIView *view in self.subviews) {
        if([view isKindOfClass:[ActivityView class]]) {
            [(ActivityView *)view stopAnimating];
            [view removeFromSuperview];
            return;
        }
    }
}

- (void)loadImageViewURL:(NSString *)urlString
{
    NSString *encodedString = [urlString md5];
    NSString *path = [NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path = [path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.png",encodedString]];
    __weak UIImageView *weakSelf = self;
    if([[NSFileManager defaultManager]fileExistsAtPath:path]) {
        UIImage *image = [UIImage imageWithContentsOfFile:path];
        dispatch_async(dispatch_get_main_queue(), ^{
            __strong UIImageView *strongSelf = weakSelf;
            if(strongSelf) {
                strongSelf.image = image;
            }
        });
    }
    else {
        [self startLoadImageFromNetwork];
        NSURL *url = [NSURL URLWithString:urlString];
        
        ImageDownloader *imageDownloader =[[ImageDownloader alloc] initWithImageURL: url];
        
        ImageDownloaderSuccessCallback successCallback = ^(id downloader, UIImage *image)
        {
            __weak UIImageView *weakSelf = self;
            dispatch_async(dispatch_get_main_queue(), ^{
                __strong UIImageView *strongSelf = weakSelf;
                NSData *response = UIImagePNGRepresentation(image);
                [response writeToFile:path atomically:YES];
                if(strongSelf) {
                    strongSelf.image = image;
                }
                [strongSelf removeLoadingView];
            });
        };
        imageDownloader.successCallback = successCallback;
        
        __weak UIImageView *weakSelf = self;
        ImageDownloaderFailureCallback failureCallback = ^(id downloader, NSError *error)
        {
            __strong UIImageView *strongSelf = weakSelf;
            if(strongSelf) {
                strongSelf.image = nil;
            }
            [strongSelf removeLoadingView];
            
        };
        
        imageDownloader.failureCallback = failureCallback;
        
        [imageDownloader startDownload];
    }
}

@end