//
//  ConversationsViewController.m
//  TeKtoTut
//
//  Created by Admin on 16.12.12.
//
//

#import "ConversationsViewController.h"

//#import "MessageManager.h"
#import "Conversation.h"
#import "ConversationTableViewCell.h"
#import "APIRequest.h"
#import "NSString+MD5.h"
#import "ImageDownloader.h"
#import "ObjectsList.h"
#import "MessagesViewController.h"
#import "ProfileViewController.h"
#import "AppDelegate.h"
#import "MenuContainerViewController.h"

@implementation ConversationsViewController
{
    NSMutableArray *    _loadingImageURLs;
    ObjectsList *       _objectList;
}

#pragma mark - Actions
- (IBAction)nextPageButtonAction:(id)sender
{
    [_objectList loadObjectsForNextPageWithParameters:nil];
}

- (IBAction)menuButtonAction:(id)sender
{
    [[AppDelegate delegate].menuController showMenu];
}

#pragma mark - working methods
- (void)hideFooterViewIfNeed
{
    if(![_objectList isNextObjectsPageAvailable]) {
        _tableView.tableFooterView = nil;
    }
    else if(!_tableView.tableFooterView) {
        _tableView.tableFooterView = _tableFooterView;
    }
}

- (void)createLeftButton
{
    UIImage *normalImage = [UIImage imageNamed:@"button.png"];
    UIImage *pushedImaged = [UIImage imageNamed:@"button_pushed.png"];
    
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.frame = CGRectMake(0, 0, normalImage.size.width, normalImage.size.height);
    [button setBackgroundImage:normalImage forState:UIControlStateNormal];
    [button setBackgroundImage:pushedImaged forState:UIControlStateHighlighted];
    [button addTarget:self action:@selector(menuButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    
    UIBarButtonItem *item = [[UIBarButtonItem alloc]initWithCustomView:button];
    self.navigationItem.leftBarButtonItem = item;
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [self createLeftButton];
    _loadingImageURLs = [NSMutableArray array];
    _objectList = [[ObjectsList alloc]initWithAPIMethod:kDialogsRequestURL objectsCount:20];
    _objectList.delegate = (id<ObjectsListDelegate>)self;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self updateContent];
    [self hideFooterViewIfNeed];
}

- (void)dealloc
{
    _objectList.delegate = nil;
}

#pragma mark - EGORefreshTableViewMethods
- (void)updateContent
{
    NSString *serverAccessToken = [[NSUserDefaults standardUserDefaults]objectForKey:kServerAccessTokenKey];
    if(serverAccessToken) {
        [super updateContent];
        [_objectList updateObjectsWithParameters:nil];
    }
    else {
        [self performSelector:@selector(loadingFinished) withObject:nil afterDelay:0.5];
        //TODO: need login message
    }
}

#pragma mark - UITableView DataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _objectList.loadedObjects.count;
}

- (float)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 80.0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *kReuseIdentifier = @"Reuse Identifier";
    ConversationTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kReuseIdentifier];
    if(!cell) {
        cell = [[ConversationTableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kReuseIdentifier];
        cell.accessoryType = UITableViewCellAccessoryDetailDisclosureButton;
    }
    Conversation *conv = [_objectList.loadedObjects objectAtIndex:indexPath.row];
    [cell setupCellForConversation:conv];
    [cell setPeoplImage:[self getImageForConversation:conv forIndexPath:indexPath]];
    return cell;
}

#pragma mark - UITableViewDelegate
- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath
{
    Conversation *conv = [_objectList.loadedObjects objectAtIndex:indexPath.row];
    ProfileViewController *controller = [[ProfileViewController alloc]initWithPeopleID:conv.peopleID];
    [self.navigationController pushViewController:controller animated:YES];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    Conversation *conv = [_objectList.loadedObjects objectAtIndex:indexPath.row];
    MessagesViewController *controller = [[MessagesViewController alloc]initWithDialogUserID:conv.peopleID];
    [self.navigationController pushViewController:controller animated:YES];
}

#pragma mark - ObjectsListDelegate
- (void)didRecieveResponse:(id) response withError:(NSError *)error
{
    if(!error) {
        NSDictionary *responseDictionary = (NSDictionary *)response;
        
        NSArray *convArray = [responseDictionary objectForKey:@"response"];
        for(NSDictionary *dictionary in convArray) {
            Conversation *conv = [[Conversation alloc]initWithDictionary:dictionary];
            [_objectList.loadedObjects addObject:conv];
        }
    }
    else {
        //TODO: some code here
    }
    [self loadingFinished];
}

- (void)loadingFinished
{
    [super contentUpdated];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self hideFooterViewIfNeed];
        [_tableView reloadData];
    });
}

#pragma mark - image request methods
- (UIImage *)getImageForConversation:(Conversation *)conversation forIndexPath:(NSIndexPath *)indexPath
{
    NSString *urlString = conversation.peopleImageURL;
    NSString *encodedURL = [urlString md5];
    NSString *path = [NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path = [path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.png",encodedURL]];
    UIImage *image = nil;
    if([[NSFileManager defaultManager]fileExistsAtPath:path]) {
        image = [UIImage imageWithContentsOfFile:path];
    }
    else {
        if(![_loadingImageURLs containsObject:urlString]) {
            ImageDownloader *downloader = [[ImageDownloader alloc]initWithImageURL:[NSURL URLWithString:urlString]];
            downloader.successCallback = ^(ImageDownloader *downloader, UIImage *image){
                NSData *data = UIImagePNGRepresentation(image);
                [data writeToFile:path atomically:YES];
                dispatch_async(dispatch_get_main_queue(), ^{
                    if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                        UITableViewCell *cell = [_tableView cellForRowAtIndexPath:indexPath];
                        if([cell isKindOfClass:[ConversationTableViewCell class]]) {
                            ConversationTableViewCell *convCell = (ConversationTableViewCell *)cell;
                            [convCell setPeoplImage:image];
                        }
                    }
                });
                [_loadingImageURLs removeObject:urlString];
            };
            downloader.failureCallback = ^(ImageDownloader *downloader, NSError *error){
                [_loadingImageURLs removeObject:urlString];
            };
            [downloader startDownload];
            [_loadingImageURLs addObject:urlString];
        }
    }
    return image;
}
@end
