//
//  MenuContainerViewController.m
//  TeKtoTut
//
//  Created by mave on 1/30/13.
//
//

#import "MenuContainerViewController.h"

#import "NavigationController.h"

@implementation MenuContainerViewController
{
    float           _beginOriginX;
    float           _beginMenuOriginX;
    UIView *        _contentCoverView;
    NSIndexPath *   _selectedIndexPath;
}
#pragma mark - initialization
- (void)customInitialization
{
    _menuViewController = [[MenuViewController alloc]initWithNibName:@"MenuViewController" bundle:nil];
    _menuViewController.delegate = self;
    _menuViewController.view.frame = CGRectMake(-_menuViewController.view.frame.size.width, 0, _menuViewController.view.frame.size.width, [UIScreen mainScreen].bounds.size.height);
    
    [self.view addSubview:_menuViewController.view];
    [self addChildViewController:_menuViewController];
    
    [_menuViewController showViewControllerWithType:ViewControllerTypePlace];
}

- (id)init
{
    self = [super init];
    if(self) {
        [self customInitialization];
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if(self) {
        [self customInitialization];
    }
    return self;
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if(self) {
        [self customInitialization];
    }
    return self;
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    UIPanGestureRecognizer *panRecognizer = [[UIPanGestureRecognizer alloc]initWithTarget:self action:@selector(panGestureAction:)];
    panRecognizer.delegate = self;
    [self.view addGestureRecognizer:panRecognizer];
}

#pragma mark - GestureRecognizer methods
- (void)panGestureAction:(UIPanGestureRecognizer *)recognizer
{
    UIView *menuView = _menuViewController.view;
    switch (recognizer.state) {
        case UIGestureRecognizerStateBegan: {
            _beginOriginX = _contentViewController.view.frame.origin.x;
            _beginMenuOriginX = menuView.frame.origin.x;
            [_menuViewController viewWillAppear:YES];
        }
            break;
        case UIGestureRecognizerStateChanged: {
            CGPoint transPoint = [recognizer translationInView:_contentViewController.view];
            CGPoint origin = CGPointMake(_beginOriginX + transPoint.x, _contentViewController.view.frame.origin.y);
            CGPoint menuOrigin = CGPointMake(_beginMenuOriginX + transPoint.x, menuView.frame.origin.y);
            if(menuOrigin.x <= 0 && menuOrigin.x >= -_menuViewController.view.frame.size.width) {
                [UIView animateWithDuration:0.25 animations:^{
                    _contentViewController.view.frame = CGRectMake(origin.x, origin.y, _contentViewController.view.frame.size.width, _contentViewController.view.frame.size.height);
                    menuView.frame = CGRectMake(menuOrigin.x, menuOrigin.y, menuView.frame.size.width, menuView.frame.size.height);
                }];
            }
        }
            break;
            
        default: {
            if(menuView.frame.origin.x > -_menuViewController.view.frame.size.width / 2.0) {
                [self showMenu];
            }
            else {
                [self hideMenu];
            }
        }
            break;
    }
}

#pragma mark - working methods

- (void)showMenu
{
    UIView *menuView = _menuViewController.view;
    CGRect frame = CGRectMake(_menuViewController.view.frame.size.width, _contentViewController.view.frame.origin.y, _contentViewController.view.frame.size.width, _contentViewController.view.frame.size.height);
    CGRect menuFrame = CGRectMake(0, menuView.frame.origin.y, menuView.frame.size.width, menuView.frame.size.height);
    if(!_contentCoverView) {
        _contentCoverView = [[UIView alloc] initWithFrame:_contentViewController.view.bounds];
        _contentCoverView.backgroundColor = [UIColor blackColor];
        [_contentViewController.view addSubview:_contentCoverView];
    }
    _contentCoverView.alpha = 0.0;
    [UIView animateWithDuration:0.25 animations:^{
        _contentViewController.view.frame = frame;
        menuView.frame = menuFrame;
        _contentCoverView.alpha = 0.7;
    }];
}

- (void)hideMenu
{
    UIView *menuView = _menuViewController.view;
    CGRect frame = CGRectMake(0, _contentViewController.view.frame.origin.y, _contentViewController.view.frame.size.width, _contentViewController.view.frame.size.height);
    CGRect menuFrame = CGRectMake(-_menuViewController.view.frame.size.width, menuView.frame.origin.y, menuView.frame.size.width, menuView.frame.size.height);
    
    [_contentCoverView removeFromSuperview];
    _contentCoverView = nil;
    [UIView animateWithDuration:0.25 animations:^{
        _contentViewController.view.frame = frame;
        menuView.frame = menuFrame;
    }];
}

#pragma mark - menu delegate
- (void)didSelectViewController:(UIViewController *)controller;
{
    if([controller isKindOfClass:[NavigationController class]]) {
        [(NavigationController *)controller popToRootViewControllerAnimated:NO];
    }
    
    [_contentCoverView removeFromSuperview];
    _contentCoverView = nil;
    
    CGRect frame = _contentViewController.view.frame;
    
    [_contentViewController.view removeFromSuperview];
    [_contentViewController removeFromParentViewController];
    _contentViewController = nil;
    
    _contentViewController = controller;
    [self.view addSubview:_contentViewController.view];
    [self addChildViewController:_contentViewController];
    _contentViewController.view.frame = frame;
    
    [UIView animateWithDuration:0.25 animations:^{
        _menuViewController.view.frame = CGRectMake(-_menuViewController.view.frame.size.width, 0, _menuViewController.view.frame.size.width, [UIScreen mainScreen].bounds.size.height);
        _contentViewController.view.frame = self.view.bounds;
        
    }];
}

#pragma mark - UIGestureRecognizerDelegate
- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldRecognizeSimultaneouslyWithGestureRecognizer:(UIGestureRecognizer *)otherGestureRecognizer
{
    return NO;
}

/*#pragma mark - property methods
- (void)setSelectedIndex:(NSInteger)selectedIndex
{
    if(selectedIndex > 0 && selectedIndex < _menuViewController.viewConrollers.count) {
        UIViewController *controller = [_menuViewController.viewConrollers objectAtIndex:selectedIndex];
        [self didSelectViewController:controller];
    }
}

- (NSInteger)selectedIndex
{
    return [_menuViewController.viewConrollers indexOfObject:_contentViewController];
}

- (UIViewController *)selectedViewController
{
    return _contentViewController;
}*/
@end
