//
//  MenuViewController.m
//  TeKtoTut
//
//  Created by Admin on 19/01/2013.
//
//

#import "MenuViewController.h"

#import "AppDelegate.h"
#import "NavigationController.h"
#import "APIRequest.h"
#import "UIImageView+RemoteImage.h"
#import "People.h"
#import "NavigationController.h"
#import "PlacesViewController.h"
#import "ProfileViewController.h"
#import "ConversationsViewController.h"
#import "SavedPeoplesViewController.h"
#import "SettingsViewController.h"


static float const kMenuWidth = 250;
static NSString * const kProfileKey     = @"ProfileKey";
static NSString * const kPlaceKey       = @"PlaceKey";
static NSString * const kMessagesKey    = @"MessagesKey";
static NSString * const kPeoplesKey     = @"PeoplesKey";
static NSString * const kSettingsKey    = @"SettingsKey";

@implementation MenuViewController
{
    NSIndexPath * _selectedIndexPath;
    NSDictionary *  _menuControllers;
}
#pragma mark - initialization
- (void)viewControllersInitialization
{
    PlacesViewController *placesController = [PlacesViewController new];
    NavigationController *navController1 = [[NavigationController alloc]initWithRootViewController:placesController];
    navController1.title = NSLocalizedString(@"Места"  , @"Места");
    
    NSInteger peopleID = [[[NSUserDefaults standardUserDefaults]objectForKey:kServerUserID] integerValue];
    ProfileViewController *profileController = [[ProfileViewController alloc]initWithPeopleID:peopleID];
    NavigationController *navController2 = [[NavigationController alloc]initWithRootViewController:profileController];
    navController2.title = NSLocalizedString(@"Профиль"  , @"Профиль");
    
    ConversationsViewController *conversationsController = [ConversationsViewController new];
    NavigationController *navController3 = [[NavigationController alloc]initWithRootViewController:conversationsController];
    navController3.title = NSLocalizedString(@"Мои записки"  , @"Мои записки");
    
    SavedPeoplesViewController *peoplesController = [SavedPeoplesViewController new];
    NavigationController *navController4 = [[NavigationController alloc]initWithRootViewController:peoplesController];
    navController4.title = NSLocalizedString(@"Люди", @"Люди");
    
    SettingsViewController *settings = [SettingsViewController new];
    NavigationController *navController5 = [[NavigationController alloc]initWithRootViewController:settings];
    navController5.title = NSLocalizedString(@"Настройки", @"Настройки");
    
    _menuControllers = [NSDictionary dictionaryWithObjectsAndKeys:navController1, kPlaceKey, navController2, kProfileKey, navController3, kMessagesKey, navController4, kPeoplesKey, navController5, kSettingsKey, nil];
}

- (void)customInitialization
{
    [self viewControllersInitialization];
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if(self) {
        [self customInitialization];
    }
    return self;
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    self.wantsFullScreenLayout = NO;
    _profileView.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"bg_name_leftmenu.png"]];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    if(isUserLogin()) {
        NSInteger userID = [[[NSUserDefaults standardUserDefaults] objectForKey:kServerUserID] integerValue];
        [APIRequest requestPeopleInfoForId:userID responseHandler:^(People *people) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [_profileImageView loadImageViewURL:people.photoURL];
                _profileLabel.text = people.firstName;
            });
        }];
    }
}

#pragma mark - working methods
- (NSString *)keyForRow:(NSInteger)row
{
    switch(row) {
        case ViewControllerTypeProfile: {
            return kProfileKey;
        }
            break;
        case ViewControllerTypePlace: {
            return kPlaceKey;
        }
            break;
        case ViewControllerTypeMessages: {
            return kMessagesKey;
        }
            break;
        case ViewControllerTypePeoples: {
            return kPeoplesKey;
        }
            break;
        case ViewControllerTypeSettings: {
            return kSettingsKey;
        }
            break;
            
        default: {
            NSAssert(NO, @"Unknown row");
            return nil;
        }
            break;
    }
}

- (UIImage *)imageForControllerWithType:(ViewControllerType)type
{
    switch(type) {
        case ViewControllerTypeProfile: {
            return [UIImage imageNamed:@"user_leftmenu.png"];
        }
            break;
        case ViewControllerTypePlace: {
            return [UIImage imageNamed:@"places_leftmenu.png"];
        }
            break;
        case ViewControllerTypeMessages: {
            return [UIImage imageNamed:@"usermessage_leftmenu.png"];
        }
            break;
        case ViewControllerTypePeoples: {
            return [UIImage imageNamed:@"people.png"];
        }
            break;
        case ViewControllerTypeSettings: {
            return [UIImage imageNamed:@"setting.png"];
        }
            break;
            
        default: {
            NSAssert(NO, @"Unknown type");
            return nil;
        }
            break;
    }
}

- (void)showViewControllerWithType:(ViewControllerType)type
{
    NSString *key = [self keyForRow:type];
    UIViewController *controller = [_menuControllers objectForKey:key];
    if([_delegate respondsToSelector:@selector(didSelectViewController:)]) {
        [_delegate didSelectViewController:controller];
    }
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return ViewControllerCount;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString * const kReuseIdentifier = @"Reuse Identifier";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kReuseIdentifier];
    if(!cell) {
        cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kReuseIdentifier];
        cell.accessoryType = UITableViewCellAccessoryNone;
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.textLabel.textColor = [UIColor whiteColor];
    }
    NSString *key = [self keyForRow:indexPath.row];
    UIViewController *controller = [_menuControllers objectForKey:key];
    cell.textLabel.text = controller.title;
    cell.imageView.image = [self imageForControllerWithType:indexPath.row];
    return cell;
}

#pragma mark - UITableViewDelegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSString *key = [self keyForRow:indexPath.row];
    UIViewController *controller = [_menuControllers objectForKey:key];
    if([_delegate respondsToSelector:@selector(didSelectViewController:)]) {
        [_delegate didSelectViewController:controller];
    }
    _selectedIndexPath = indexPath;
    [tableView reloadData];
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if([indexPath isEqual:_selectedIndexPath]) {
        cell.backgroundColor = [UIColor colorWithRed:0 green:174.0 / 255.0 blue:239.0 / 255.0 alpha:1.0];
    }
    else {
        cell.backgroundColor = [UIColor clearColor];
    }
}
@end