//
//  MessagesViewController.m
//  TeKtoTut
//
//  Created by mave on 1/10/13.
//
//

#import "MessagesViewController.h"

#import "APIRequest.h"
#import "ObjectsList.h"
#import "Message.h"
#import "MessageTableViewCell.h"
#import "TextInputView.h"

@implementation MessagesViewController
{
    ObjectsList *_objectsList;
}

#pragma mark - Actions
- (IBAction)nextPageButtonAction:(id)sender
{
    [_objectsList loadObjectsForNextPageWithParameters:[self createParameters]];
}

- (void)tapGestureAction:(id)tapRecognizer
{
    if([_textIntputVIew isFirstResponder]) {
        [_textIntputVIew resignFirstResponder];
    }
}

- (IBAction)winkButtonAction:(id)sender
{
    [APIRequest sendMessageWithSenderID:_dialogUserID
                            messageText:kWinkMessage
                        responseHandler:^(BOOL success) {
                            if(success) {
                                Message *message = [[Message alloc]init];
                                message.senderID = _currentUserId;
                                message.date = [NSDate date];
                                message.text = kWinkMessage;
                                [_objectsList.loadedObjects insertObject:message atIndex:0];
                                NSIndexPath *indexPath = [NSIndexPath indexPathForRow:_objectsList.loadedObjects.count - 1 inSection:0];
                                [_tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationBottom];
                                [_tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionBottom animated:YES];
                            }
                        }
     ];
}

#pragma mark - initialization
- (id)initWithDialogUserID:(NSInteger)dialogUserID;
{
    self = [super initWithNibName:@"MessagesViewController" bundle:nil];
    if(self) {
        _dialogUserID = dialogUserID;
    }
    return self;
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    [self addNotificationObserver];
    
    _objectsList = [[ObjectsList alloc]initWithAPIMethod:kMessagesRequestURL objectsCount:30];
    _objectsList.delegate = (id<ObjectsListDelegate>)self;
    
    _currentUserId = [[[NSUserDefaults standardUserDefaults]objectForKey:kServerUserID]integerValue];
    [self hideFooterViewIfNeed];
    
    UITapGestureRecognizer *rec = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(tapGestureAction:)];
    [self.view addGestureRecognizer:rec];
    
    [self createBarButtons];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self updateContent];
}

- (void)dealloc
{
    _objectsList.delegate = nil;
    [self removeNotificationObserver];
}

#pragma mark - working methods
- (void)markMessageAsRead:(Message *)message
{
    if(![message isRead] && message.senderID != _currentUserId) {
        [APIRequest markAsReadMessageWithID:message.ID responseHandler:^(BOOL success) {
            if(success) {
                [UIApplication sharedApplication].applicationIconBadgeNumber -= 1;
            }
        }];
    }
}

- (void)createBarButtons
{
    UIBarButtonItem *wink = [[UIBarButtonItem alloc]initWithTitle:NSLocalizedString(@"Подмигнуть", @"Подмигнуть") style:UIBarButtonItemStylePlain target:self action:@selector(winkButtonAction:)];
    self.navigationItem.rightBarButtonItem = wink;
}

- (void)hideFooterViewIfNeed
{
    if(![_objectsList isNextObjectsPageAvailable]) {
        _tableView.tableHeaderView = nil;
    }
    else if(!_tableView.tableHeaderView) {
        _tableView.tableHeaderView = _tableFooterView;
    }
}

- (void)loadingFinished
{
    [super contentUpdated];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self hideFooterViewIfNeed];
        [_tableView reloadData];
        if(_objectsList.loadedObjects.count) {
            NSIndexPath *indexPath = [NSIndexPath indexPathForRow:(_objectsList.loadedObjects.count - 1) inSection:0];
            [_tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionBottom animated:YES];
        }
    });
}

- (NSString *)createParameters
{
    NSString *parameters = [NSString stringWithFormat:@"uid=%d", _dialogUserID];
    return parameters;
}

#pragma mark - EGORefreshTableViewMethods
- (void)updateContent
{
    [super updateContent];
    [_objectsList updateObjectsWithParameters:[self createParameters]];
}

#pragma mark - UITablewViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _objectsList.loadedObjects.count;
}

- (float)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    Message *message = [_objectsList.loadedObjects objectAtIndex:_objectsList.loadedObjects.count - 1 - indexPath.row];
    UIFont *font = [UIFont systemFontOfSize:14.0];
    CGSize size = [message.text sizeWithFont:font constrainedToSize:CGSizeMake(250, 100000)];
    return size.height + 20;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *kReuseIdentifier = @"Reuse Identifier";
    MessageTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kReuseIdentifier];
    if(!cell) {
        cell = [[MessageTableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kReuseIdentifier];
        cell.accessoryType = UITableViewCellAccessoryNone;
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
    }
    
    Message *message = [_objectsList.loadedObjects objectAtIndex:_objectsList.loadedObjects.count - 1 - indexPath.row];
    [cell setMessage:message forCurrentUserID:_currentUserId];
    [self markMessageAsRead:message];
    return cell;
}

#pragma mark - UITableViewDelegate


#pragma mark - ObjectsListDelegate
- (void)didRecieveResponse:(id) response withError:(NSError *)error
{
    if(!error) {
        NSDictionary *responseDictionary = (NSDictionary *)response;
        
        NSArray *convArray = [responseDictionary objectForKey:@"response"];
        for(NSDictionary *dictionary in convArray) {
            Message *mess = [[Message alloc]initWithDictionaryInfo:dictionary];
            [_objectsList.loadedObjects addObject:mess];
        }
    }
    else {
        //TODO: some code here
    }
    [self loadingFinished];
}

#pragma mark - TextInputViewDelegate
- (void)sendButtonActionWithText:(NSString *)text
{
    [APIRequest sendMessageWithSenderID:_dialogUserID messageText:text responseHandler:^(BOOL success) {
        if(success) {
            Message *message = [[Message alloc]init];
            message.senderID = _currentUserId;
            message.date = [NSDate date];
            message.text = text;
            [_objectsList.loadedObjects insertObject:message atIndex:0];
            NSIndexPath *indexPath = [NSIndexPath indexPathForRow:_objectsList.loadedObjects.count - 1 inSection:0];
            [_tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationBottom];
            [_tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionBottom animated:YES];
        }
    }];
}

#pragma mark - Notifications methods
- (void)addNotificationObserver
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleNewMessageNotification:) name:kNewMessageNotification object:nil];
}

- (void)removeNotificationObserver
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kNewMessageNotification object:nil];
}

- (void)handleNewMessageNotification:(id)notification
{
    [self updateContent];
}
@end
