//
//  PeopleProfileViewController.m
//  TeKtoTut
//
//  Created by Admin on 14/12/2012.
//
//

#import "PeopleProfileViewController.h"

#import "ActivityView.h"
#import "APIRequest.h"
#import "ImageDownloader.h"
#import "Image.h"
#import "NSString+MD5.h"
#import "People.h"
#import "PeopleLinksViewController.h"
#import "NavigationController.h"
#import "AppDelegate.h"
#import "MessagesViewController.h"
#import "MenuContainerViewController.h"
#import "ScrollPageView.h"

enum
{
    ActionSheetShowLinksAction      = 0,
    ActionSheetWriteMessageAction   = 1,
    ActionSheetCancelAction         = 2
};

@implementation PeopleProfileViewController
{
    NSMutableArray *    _images;
    NSMutableArray *    _loadingRequests;
    NSInteger           _startIndex;
}

#pragma mark - Actions
- (IBAction)rightButtonAction:(id)sender
{
    UIActionSheet *actionSheet = [[UIActionSheet alloc]initWithTitle:NSLocalizedString(@"Выберите действие", @"Выберите действие") delegate:self cancelButtonTitle:NSLocalizedString(@"Отмена", @"Отмена") destructiveButtonTitle:nil otherButtonTitles:NSLocalizedString(@"Показать ссылки", @"Показать ссылки"), NSLocalizedString(@"Написать сообщение", @"Написать сообщение"), nil];
    actionSheet.actionSheetStyle = UIActionSheetStyleBlackOpaque;
    [actionSheet showInView:self.view];
}

- (IBAction)leftButtonAction:(id)sender
{
    [self.presentingViewController dismissModalViewControllerAnimated:YES];
}

#pragma mark - initialization
- (id)initWithPeople:(People *)people startIndex:(NSInteger)startIndex
{
    self = [super initWithNibName:@"PeopleProfileViewController" bundle:nil];
    if(self) {
        _people = people;
        _startIndex = startIndex;
    }
    return self;
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    [self createBarButtons];
    _images = [[NSMutableArray alloc]init];
    _loadingRequests = [[NSMutableArray alloc]init];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self loadProfileImages];
}

- (void)createBarButtons
{
    UIBarButtonItem *right = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemAction target:self action:@selector(rightButtonAction:)];
    self.navigationItem.rightBarButtonItem = right;
    
    UIBarButtonItem *left = [[UIBarButtonItem alloc]initWithTitle:NSLocalizedString(@"Назад", @"Назад") style:UIBarButtonItemStylePlain target:self action:@selector(leftButtonAction:)];
    self.navigationItem.leftBarButtonItem = left;
}

- (void)loadProfileImages
{
    ActivityView *view = [[ActivityView alloc]init];
    [self.view addSubview:view];
    view.center = CGPointMake(CGRectGetMidX(self.view.bounds), CGRectGetMidY(self.view.bounds));
    [view startAnimating];
    [APIRequest requestPeopleImagesForId:_people.peopleID responseHandler:^(NSArray *images) {
        [_images removeAllObjects];
        [_images addObjectsFromArray:images];
        dispatch_async(dispatch_get_main_queue(), ^{
            [view stopAnimating];
            [view removeFromSuperview];
            [_scrollView reloadContent];
            [_scrollView setPage:_startIndex];
        });
    }];
}

#pragma mark - ScrollPageView DataSource
- (NSInteger)numberOfPages
{
    return _images.count;
}

- (UIView *)viewForIndex:(NSInteger)index
{
    UIImageView *imageView = [[UIImageView alloc]init];
    imageView.contentMode = UIViewContentModeScaleAspectFit;
    [self loadImageWithIndex:index inView:imageView];
    return imageView;
}


- (void)loadImageWithIndex:(NSInteger)imageIndex inView:(UIImageView *)imageView
{
    imageView.image = nil;
    Image *profileImage = [_images objectAtIndex:imageIndex];
    NSString *urlString = profileImage.imageURL;
    NSString *encodedURL = [urlString md5];
    NSString *path = [NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path = [path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.png",encodedURL]];
    if([[NSFileManager defaultManager]fileExistsAtPath:path]) {
        imageView.image = [UIImage imageWithContentsOfFile:path];
    }
    else {
        imageView.image = nil;
        if(![_loadingRequests containsObject:urlString]) {
            ImageDownloader *downloader = [[ImageDownloader alloc]initWithImageURL:[NSURL URLWithString:urlString]];
            downloader.successCallback = ^(ImageDownloader *downloader, UIImage *image){
                dispatch_async(dispatch_get_main_queue(), ^{
                    //TODO: add image updating logic
//                    NSInteger loadedImageIndex = [_images indexOfObject:profileImage];
//                    [self setupImageViewWithImage:image index:loadedImageIndex];
                });
                NSData *data = UIImagePNGRepresentation(image);
                [data writeToFile:path atomically:YES];
                [_loadingRequests removeObject:urlString];
            };
            downloader.failureCallback = ^(ImageDownloader *downloader, NSError *error){
                [_loadingRequests removeObject:urlString];
                dispatch_async(dispatch_get_main_queue(), ^{
                });
            };
            [downloader startDownload];
            [_loadingRequests addObject:urlString];
        }
    }
}

/*#pragma mark - Activity view methods
- (void)showActivityInView:(UIView *)view
{
    if(!view.subviews.count) {
        ActivityView *activityView = [[ActivityView alloc]init];
        [view addSubview:activityView];
        activityView.center = CGPointMake(CGRectGetMidX(view.bounds), CGRectGetMidY(view.bounds));
        [activityView startAnimating];
    }
}

- (void)hideActivityFromView:(UIView *)view
{
    [view.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
}*/

#pragma mark - Action Sheet Delegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    switch (buttonIndex) {
        case ActionSheetShowLinksAction: {
            PeopleLinksViewController *controller = [[PeopleLinksViewController alloc]initWithLinks:_people.links];
            [self.navigationController pushViewController:controller animated:YES];
        }
            break;
        case ActionSheetWriteMessageAction: {
            NSInteger currentUserId = [[[NSUserDefaults standardUserDefaults]objectForKey:kServerUserID]integerValue];
            if(currentUserId != _people.peopleID) {
                MessagesViewController *mesController = [[MessagesViewController alloc]initWithDialogUserID:_people.peopleID];
                [self.navigationController pushViewController:mesController animated:YES];
            }
        }
            break;
        case ActionSheetCancelAction: {
            NSLog(@"Cancel");
        }
            break;
            
        default: {
            NSAssert(NO, @"Unknown action sheet action");
        }
            break;
    }
}
@end
