//
//  PeoplesViewController.m
//  TeKtoTut
//
//  Created by mave on 1/23/13.
//
//

#import "PeoplesViewController.h"

#import "APIRequest.h"
#import "People.h"
#import "ProfileViewController.h"
#import "ImageLoader.h"
#import "PeopleMainInfoCell.h"

@implementation PeoplesViewController
{
    ImageLoader *_imageLoader;
}

#pragma mark - initialization
- (id)initWithPeoples:(NSArray *)peoples
{
    self = [super initWithNibName:@"PeoplesViewController" bundle:nil];
    if(self) {
        _peoples = [NSArray arrayWithArray:peoples];
    }
    return self;
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    _imageLoader = [[ImageLoader alloc]initWithDelegate:(id<ImageLoaderDelegate>)self];
}

- (void)dealloc
{
    _imageLoader.delegate = nil;
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _peoples.count;
}

- (float)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 70.0f;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString * const reuseIdentifier = @"PeopleReuseIdentifier";
    PeopleMainInfoCell *cell = [_tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
    if(!cell) {
        cell = [[PeopleMainInfoCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
        cell.selectionStyle = UITableViewCellSelectionStyleBlue;
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    People *people = [_peoples objectAtIndex:indexPath.row];
    [cell setViewForPeople:people];
    
    [_imageLoader loadImageForURL:people.photoURL atIndexPath:indexPath];
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    [self showProfileViewControllerForPeopleAtIndex:indexPath.row];
}

#pragma mark - navigation method
- (void)showProfileViewControllerForPeopleAtIndex:(NSInteger)index
{
    People *localPeople = [_peoples objectAtIndex:index];
    [APIRequest requestPeopleInfoForId:localPeople.peopleID responseHandler:^(People *people) {
        dispatch_async(dispatch_get_main_queue(), ^{
            ProfileViewController *controller = [[ProfileViewController alloc]initWithPeopleID:people.peopleID];
            [self.navigationController pushViewController:controller animated:YES];
        });
    }];
}

#pragma mark - ImageLoadeDelegate
- (void)didLoadImage:(UIImage *)image forPath:(NSArray *)indexPaths
{
    dispatch_async(dispatch_get_main_queue(), ^{
        for(NSIndexPath *indexPath in indexPaths) {
            if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                PeopleMainInfoCell *cell = (PeopleMainInfoCell *)[_tableView cellForRowAtIndexPath:indexPath];
                [cell setPeopleImage:image];
            }
        }
    });
}

- (void)didFailLoadingImageForPath:(NSArray *)indexPaths
{
    dispatch_async(dispatch_get_main_queue(), ^{
        for(NSIndexPath *indexPath in indexPaths) {
            if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                PeopleMainInfoCell *cell = (PeopleMainInfoCell *)[_tableView cellForRowAtIndexPath:indexPath];
                [cell setPeopleImage:nil];
            }
        }
    });
}
@end
