//
//  PlaceDetailViewController.m
//  TeKtoTut
//
//  Created by Admin on 02.12.12.
//
//

#import "PlaceDetailViewController.h"

#import "APIRequest.h"
#import "People.h"
#import "Place.h"
#import "PlaceCell.h"
#import "ProfileViewController.h"
#import "PeopleMainInfoCell.h"
#import "ImageLoader.h"
#import "PartsControl.h"
#import "PeopleStatusCell.h"

typedef enum {
    ScreenStatePeople = 0,
    ScreenStateInfo = 1
}
ScreenState;

enum {
    InfoRowPlaceCell = 0,
    InfoRowTextCell,
    InfoRowsCount
};

@implementation PlaceDetailViewController
{
    NSMutableArray *            _peoples;
    ScreenState                 _screenState;
    PlaceCell *                 _placeCell;
    NSMutableArray *            _loadingRequests;
    ImageLoader *               _imageLoader;
    PartsControl *              _partsControl;
}

#pragma mark - Actions
- (IBAction)partsControleValueChanged:(PartsControl *)sender
{
    _screenState = sender.selectionState;
    [self updateContent];
}

- (IBAction)checkinButtonAction:(id)sender
{
    [APIRequest checkinRequestForPlace:_place responseHandler:^{
        dispatch_async(dispatch_get_main_queue(), ^{
            [self updateContent];
        });
    }];
}

- (IBAction)actionButtonTaped:(id)sender
{
    NSString *title = NSLocalizedString(@"Выберите действие", @"Выберите действие");
    NSString *cancelTitle = NSLocalizedString(@"Отмена", @"Отмена");
    NSString *setPeopleTitle = NSLocalizedString(@"Сохранить людей места", @"Сохранить людей места");
    UIActionSheet *sheet = [[UIActionSheet alloc]initWithTitle:title delegate:self cancelButtonTitle:cancelTitle destructiveButtonTitle:nil otherButtonTitles:setPeopleTitle, nil];
    [sheet showInView:self.view];
}

#pragma mark - initialization
- (id)initWithPlace:(Place *)place showPlaceInfo:(BOOL)showPlaceInfo
{
    self = [super initWithNibName:@"PlaceDetailViewController" bundle:nil];
    if(self) {
        _place = place;
        [self.view setNeedsDisplay];
        if(!showPlaceInfo) {
            _tableView.tableHeaderView = nil;
        }
    }
    return self;
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    [_checkinButton setTitle:NSLocalizedString(@"Я тут", @"Я тут") forState:UIControlStateNormal];
    [self createBarButtons];
    [self createPartsControl];
    [self createPlaceCell];
    _loadingRequests = [NSMutableArray array];
    _peoples = [[NSMutableArray alloc]init];
    _screenState = ScreenStatePeople;
    _imageLoader = [[ImageLoader alloc]initWithDelegate:(id<ImageLoaderDelegate>)self];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    [self updateContent];
    if(![self.navigationController.navigationBar.subviews containsObject:_partsControl]) {
        [self.navigationController.navigationBar addSubview:_partsControl];
    }
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [_partsControl removeFromSuperview];
}

- (void)dealloc
{
    _imageLoader.delegate = nil;
}

#pragma mark - working methods
- (void)createPartsControl
{
    _partsControl = [[PartsControl alloc]init];
    _partsControl.center = CGPointMake(160, 22);
    _partsControl.leftTitle = NSLocalizedString(@"Люди", @"Люди");
    _partsControl.rightTitle = NSLocalizedString(@"Инфо", @"Инфо");
    _partsControl.selectionState = PartsControlSelectionStateLeft;
    [_partsControl addTarget:self action:@selector(partsControleValueChanged:) forControlEvents:UIControlEventValueChanged];
}

- (void)createBarButtons
{
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.frame = CGRectMake(0, 0, 35, 30);
    [button setImage:[UIImage imageNamed:@"heart+.png"] forState:UIControlStateNormal];
    [button addTarget:self action:@selector(actionButtonTaped:) forControlEvents:UIControlEventTouchUpInside];
    _actionsButton = [[UIBarButtonItem alloc]initWithCustomView:button];
    self.navigationItem.rightBarButtonItem = _actionsButton;
}

#pragma mark - EGORefreshTableViewMethods
- (void)updateContent
{
    [super updateContent];
    [APIRequest requestPeopleForPlace:_place responseHandler:^(NSArray *peoples) {
        if(_screenState == ScreenStatePeople) {
            [super contentUpdated];
        }
        [_peoples removeAllObjects];
        [_peoples addObjectsFromArray:peoples];
        dispatch_async(dispatch_get_main_queue(), ^{
            [_tableView reloadData];
        });
    }];
    [APIRequest updatePlaceInfoForPlace:_place responseHandler:^{
        if(_screenState == ScreenStateInfo) {
            [self placeUpdated];
        }
        _checkinButton.enabled = !_place.checkin;
        _actionsButton.enabled = _place.checkin;
    }];
}

#pragma mark - PlaceCell methods
- (void)createPlaceCell
{
    _placeCell = (PlaceCell *)[[[NSBundle mainBundle]loadNibNamed:@"PlaceCell" owner:nil options:nil] objectAtIndex:0];
    [_placeCell setupCellForPlace:_place];
    [_placeCell loadImageForPlace:_place];
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if(_screenState == ScreenStatePeople) {
        return _peoples.count;
    }
    else {
        return InfoRowsCount;
    }
}

- (float)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if(_screenState == ScreenStatePeople) {
        return 60.0f;
    }
    else {
        switch (indexPath.row) {
            case InfoRowPlaceCell: {
                return 250.0f;
            }
                break;
            case InfoRowTextCell: {
                PeopleStatusCell *cell = (PeopleStatusCell *)[self tableView:tableView cellForRowAtIndexPath:indexPath];
                NSString *status =_place.info;
                CGSize size = [status sizeWithFont:cell.peopleStatusLabel.font constrainedToSize:CGSizeMake(cell.peopleStatusLabel.frame.size.width, 1000)];
                return size.height + 31;

            }
                break;
            default: {
                NSAssert(NO, @"unknown place info cell");
                return 0;
            }
                break;
        }
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if(_screenState == ScreenStatePeople) {
        return [self tableViewCellForPeopleAtIndexPath:indexPath];;
    }
    else {
        return [self tableViewCellForPlaceInfoAtIndexPath:indexPath];
    }
}

- (UITableViewCell *)tableViewCellForPeopleAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString * const reuseIdentifier = @"PeopleReuseIdentifier";
    PeopleMainInfoCell *cell = [_tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
    if(!cell) {
        cell = [[PeopleMainInfoCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    People *people = [_peoples objectAtIndex:indexPath.row];
    [cell setViewForPeople:people];
    [_imageLoader loadImageForURL:people.photoURL atIndexPath:indexPath];
    return cell;
}

- (UITableViewCell *)tableViewCellForPlaceInfoAtIndexPath:(NSIndexPath *)indexPath
{
    switch (indexPath.row) {
        case InfoRowPlaceCell: {
            return _placeCell;
        }
            break;
        case InfoRowTextCell: {
            static NSString * const reuseIdentifier = @"PeopleInfoStatusReuseIdentifier";
            PeopleStatusCell *cell = [_tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
            if(!cell) {
                cell = [[PeopleStatusCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
                cell.selectionStyle = UITableViewCellSelectionStyleNone;
                cell.accessoryType = UITableViewCellAccessoryNone;
            }
            NSString *status = _place.info;
            [cell setStatus:status];
            return cell;
        }
            break;
            
        default: {
            NSAssert(NO, @"unknown place info cell");
            return nil;
        }
            break;
    }
}

#pragma mark - UITableViewDelegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    if(_screenState == ScreenStatePeople) {
        [self showProfileViewControllerForPeopleAtIndex:indexPath.row];
    }
}

- (void)placeUpdated
{
    [super contentUpdated];
    dispatch_async(dispatch_get_main_queue(), ^{
        [_placeCell setupCellForPlace:_place];
        [_placeCell loadImageForPlace:_place];
        [_tableView reloadData];
    });
}

#pragma mark - navigation method
- (void)showProfileViewControllerForPeopleAtIndex:(NSInteger)index
{
    People *localPeople = [_peoples objectAtIndex:index];
    [self showActivityView];
    [APIRequest requestPeopleInfoForId:localPeople.peopleID responseHandler:^(People *people) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self hideActivityView];
            ProfileViewController *controller = [[ProfileViewController alloc]initWithPeopleID:people.peopleID];
            [self.navigationController pushViewController:controller animated:YES];
        });
    }];
}

#pragma mark - ImageLoadeDelegate
- (void)didLoadImage:(UIImage *)image forPath:(NSArray *)indexPaths
{
    dispatch_async(dispatch_get_main_queue(), ^{
        for(NSIndexPath *indexPath in indexPaths) {
            if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                PeopleMainInfoCell *cell = (PeopleMainInfoCell *)[_tableView cellForRowAtIndexPath:indexPath];
                [cell setPeopleImage:image];
            }
        }
    });
}

- (void)didFailLoadingImageForPath:(NSArray *)indexPaths
{
    dispatch_async(dispatch_get_main_queue(), ^{
        for(NSIndexPath *indexPath in indexPaths) {
            if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                PeopleMainInfoCell *cell = (PeopleMainInfoCell *)[_tableView cellForRowAtIndexPath:indexPath];
                [cell setPeopleImage:nil];
            }
        }
    });
}

#pragma mark - ActionSheet delegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    switch (buttonIndex) {
        case 0: {
            //save people action
            [APIRequest savePlaceGuests:_place.placeID];
        }
            break;
        case 1: {
            //cancel action
        }
            break;
            
        default: {
            NSAssert(NO, @"Unknown button index in action sheet");
        }
            break;
    }
}
@end