//
//  PlacesViewController.m
//  TeKtoTut
//
//  Created by Admin on 01.12.12.
//
//

#import "PlacesViewController.h"

#import "ObjectsList.h"
#import "Place.h"
#import "LocationManager.h"
#import "PlaceCell.h"
#import "APIRequest.h"
#import "PlaceDetailViewController.h"
#import "ImageLoader.h"
#import "PartsControl.h"
#import "AppDelegate.h"
#import "MenuContainerViewController.h"

/*typedef enum {
    PlacesSortTypeDistance = 1,
    PlacesSortTypeTop = 0
}PlacesSortType;*/


@implementation PlacesViewController
{
    ObjectsList *               _objectsList;
    BOOL                        _isLocationUpdated;
    ImageLoader *               _imageLoader;
    PartsControl *              _partsControl;
}

#pragma mark - Actions
- (IBAction)partsControleValueChanged:(PartsControl *)sender
{
    [self updateContent];
}

- (IBAction)nextPageButtonAction:(id)sender
{
    [_objectsList loadObjectsForNextPageWithParameters:[self createParameters]];
}

- (IBAction)menuButtonAction:(id)sender
{
    [[AppDelegate delegate].menuController showMenu];
}

#pragma mark - working methods
- (void)configurePartsControll
{
    _partsControl.leftTitle = NSLocalizedString(@"Топ", @"Топ");
    _partsControl.rightTitle = NSLocalizedString(@"Ближайшие", @"Ближайшие");
    if(![LocationManager locationServicesEnabled]){
        _partsControl.selectionState = PartsControlSelectionStateLeft;
        _partsControl.enabled = NO;
    }
}

- (void)hideFooterViewIfNeed
{
    if(![_objectsList isNextObjectsPageAvailable]) {
        _tableView.tableFooterView = nil;
    }
    else if(!_tableView.tableFooterView) {
        _tableView.tableFooterView = _tableFooterView;
    }
}

- (NSString *)createParameters
{
    NSString *parameters = @"density=240";
    if (_partsControl.selectionState == PartsControlSelectionStateRight) {
        if([LocationManager locationServicesEnabled]) {
            CLLocationCoordinate2D coordinate = [LocationManager currentLocation].coordinate;
            parameters = [parameters stringByAppendingFormat:@"&lat=%f&lng=%f",coordinate.latitude,coordinate.longitude];
        }
    }
    return parameters;
}

- (void)createPartsControl
{
    _partsControl = [[PartsControl alloc]init];
    _partsControl.center = CGPointMake(160, 22);
    [_partsControl addTarget:self action:@selector(partsControleValueChanged:) forControlEvents:UIControlEventValueChanged];
}

- (void)createLeftButton
{
    UIImage *normalImage = [UIImage imageNamed:@"button.png"];
    UIImage *pushedImaged = [UIImage imageNamed:@"button_pushed.png"];
    
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.frame = CGRectMake(0, 0, normalImage.size.width, normalImage.size.height);
    [button setBackgroundImage:normalImage forState:UIControlStateNormal];
    [button setBackgroundImage:pushedImaged forState:UIControlStateHighlighted];
    [button addTarget:self action:@selector(menuButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    
    UIBarButtonItem *item = [[UIBarButtonItem alloc]initWithCustomView:button];
    self.navigationItem.leftBarButtonItem = item;
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    _objectsList = [[ObjectsList alloc]initWithAPIMethod:kPlacesRequestURL objectsCount:20];
    _objectsList.delegate = (id<ObjectsListDelegate>)self;
    _imageLoader = [[ImageLoader alloc]initWithDelegate:(id<ImageLoaderDelegate>)self];
    [self registerPlaceCellNib];
    [self addNotificationsObserver];
    [self createPartsControl];
    [self createLeftButton];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    [self configurePartsControll];
    [self hideFooterViewIfNeed];
    [self updateContent];
    if(![self.navigationController.navigationBar.subviews containsObject:_partsControl]) {
        [self.navigationController.navigationBar addSubview:_partsControl];
    }
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [_partsControl removeFromSuperview];
}

- (void)dealloc
{
    [self removeNotificationsObserver];
    _objectsList.delegate = nil;
    _imageLoader.delegate = nil;
}

#pragma mark - EGORefreshTableViewMethods
- (void)updateContent
{
    [super updateContent];
    [_objectsList updateObjectsWithParameters:[self createParameters]];
}

#pragma mark - PlaceCell methods
- (void)registerPlaceCellNib
{
    UINib *nib = [UINib nibWithNibName:@"PlaceCell" bundle:nil];
    [_tableView registerNib:nib forCellReuseIdentifier:[self placeCellReuseIdentifier]];
}

- (NSString *)placeCellReuseIdentifier
{
    static NSString * const reuse = @"PlaceCellReuseIdentifier";
    return reuse;
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _objectsList.loadedObjects.count;
}

- (float)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 250.0f;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    PlaceCell *placeCell = [_tableView dequeueReusableCellWithIdentifier:[self placeCellReuseIdentifier]];
    Place *place = [_objectsList.loadedObjects objectAtIndex:indexPath.row];
    [placeCell setupCellForPlace:place];
    
    NSString *urlString = [NSString stringWithFormat:@"%@%@",ServerAddress, place.imageURL];
    [_imageLoader loadImageForURL:urlString atIndexPath:indexPath];
    
    return placeCell;
}

#pragma mark - UITableViewDelegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    Place *place = [_objectsList.loadedObjects objectAtIndex:indexPath.row];
    PlaceDetailViewController *controller = [[PlaceDetailViewController alloc]initWithPlace:place showPlaceInfo:YES];
    [self.navigationController pushViewController:controller animated:YES];
}

#pragma mark - Notifications methods
- (void)addNotificationsObserver
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(locationUpdated:) name:kUserLocationUpdated object:nil];
}

- (void)removeNotificationsObserver
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kUserLocationUpdated object:nil];
}

- (void)locationUpdated:(id)notification
{
    if(!_isLocationUpdated) {
        _isLocationUpdated = YES;
        [self updateContent];
    }
    [self configurePartsControll];
}

#pragma mark - ObjectsListDelegate
- (void)didRecieveResponse:(id) response withError:(NSError *)error
{
    if(!error) {
        NSDictionary *placesDictionary = (NSDictionary *)response;
        NSArray *placesArray = [placesDictionary objectForKey:@"response"];
        for(NSDictionary *dictionary in placesArray) {
            Place *place = [[Place alloc]initWithDictionary:dictionary];
            [_objectsList.loadedObjects addObject:place];
        }
    }
    else {
        //TODO: some code here
    }
    
    [super contentUpdated];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self hideFooterViewIfNeed];
        [_tableView reloadData];
    });
}

#pragma mark - ImageLoadeDelegate
- (void)didLoadImage:(UIImage *)image forPath:(NSArray *)indexPaths
{
    dispatch_async(dispatch_get_main_queue(), ^{
        for(NSIndexPath *indexPath in indexPaths) {
            if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                PlaceCell *cell = (PlaceCell *)[_tableView cellForRowAtIndexPath:indexPath];
                [cell setPlaceImage:image];
            }
        }
    });
}

- (void)didFailLoadingImageForPath:(NSArray *)indexPaths
{
    dispatch_async(dispatch_get_main_queue(), ^{
        for(NSIndexPath *indexPath in indexPaths) {
            if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                PlaceCell *cell = (PlaceCell *)[_tableView cellForRowAtIndexPath:indexPath];
                [cell setPlaceImage:nil];
            }
        }
    });
}
@end