//
//  ProfileViewController.m
//  TeKtoTut
//
//  Created by Admin on 08.12.12.
//
//

#import "ProfileViewController.h"

#import "NavigationController.h"
#import "APIRequest.h"
#import "People.h"
#import "PeopleInfoTableViewCell.h"
#import "Image.h"
#import "NSString+MD5.h"
#import "ImageDownloader.h"
#import "ImagesTableViewCell.h"
#import "VKImagesImportControllerViewController.h"
#import "AppDelegate.h"
#import "PeopleMainInfoCell.h"
#import "PeopleProfileViewController.h"
#import "PartsControl.h"
#import "PeopleStatusCell.h"
#import "MessagesViewController.h"
#import "AppDelegate.h"
#import "MenuContainerViewController.h"

typedef enum {
    ScreenStateInfo = 0,
    ScreenStateImages = 1
}
ScreenState;

enum
{
    InfoRowMain = 0,
    InfoRowStatus,
    InfoRowAdditional,
    InfoRowsCount
};

@implementation ProfileViewController
{
    ScreenState         _state;
    People *            _currentPeople;
    NSMutableArray *    _peopleImages;
    NSMutableArray *    _loadingRequests;
    Image *             _currentImage;
    PartsControl *      _partsControl;
}

#pragma mark - Actions
- (IBAction)writeMessageAction:(id)sender
{
    MessagesViewController *mesController = [[MessagesViewController alloc]initWithDialogUserID:_peopleID];
    [self.navigationController pushViewController:mesController animated:YES];
}

- (IBAction)partsControleValueChanged:(PartsControl *)sender
{
    _state = sender.selectionState;
    [self updateContent];
}

- (IBAction)rightBarButtonAction:(id)sender
{
    VKImagesImportControllerViewController *controller = [VKImagesImportControllerViewController new];
    [self.navigationController pushViewController:controller animated:YES];
}

- (IBAction)menuButtonAction:(id)sender
{
    [[AppDelegate delegate].menuController showMenu];
}

#pragma mark - initialization
- (id)initWithPeopleID:(NSInteger)peopleID;
{
    self = [super initWithNibName:@"ProfileViewController" bundle:nil];
    if(self) {
        _peopleID = peopleID;
    }
    return self;
}

#pragma mark - view customization
- (void)createRightButton
{
    NSInteger userID = [[[NSUserDefaults standardUserDefaults] objectForKey:kServerUserID] integerValue];
    if(userID == _currentPeople.peopleID) {
        UIBarButtonItem *rightButton = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(rightBarButtonAction:)];
        self.navigationItem.rightBarButtonItem = rightButton;
    }
}

- (void)createLeftButton
{
    UIImage *normalImage = [UIImage imageNamed:@"button.png"];
    UIImage *pushedImaged = [UIImage imageNamed:@"button_pushed.png"];
    
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.frame = CGRectMake(0, 0, normalImage.size.width, normalImage.size.height);
    [button setBackgroundImage:normalImage forState:UIControlStateNormal];
    [button setBackgroundImage:pushedImaged forState:UIControlStateHighlighted];
    [button addTarget:self action:@selector(menuButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    
    UIBarButtonItem *item = [[UIBarButtonItem alloc]initWithCustomView:button];
    self.navigationItem.leftBarButtonItem = item;
}

- (void)createPartsControl
{
    _partsControl = [[PartsControl alloc]init];
    _partsControl.center = CGPointMake(160, 22);
    _partsControl.leftTitle = NSLocalizedString(@"Инфо", @"Инфо");
    _partsControl.rightTitle = NSLocalizedString(@"Фото", @"Фото");
    _partsControl.selectionState = PartsControlSelectionStateLeft;
    [_partsControl addTarget:self action:@selector(partsControleValueChanged:) forControlEvents:UIControlEventValueChanged];
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    _state = ScreenStateInfo;
    _peopleImages = [NSMutableArray array];
    _loadingRequests = [NSMutableArray array];
    [self createLeftButton];
    [self createRightButton];
    [self createPartsControl];
    NSInteger currentUserId = [[[NSUserDefaults standardUserDefaults]objectForKey:kServerUserID]integerValue];
    if(currentUserId == _peopleID) {
        CGRect tableFrame = _tableView.frame;
        tableFrame.size.height += 53;
        _tableView.frame = tableFrame;
    }
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    [self updateContent];
    
    if(![self.navigationController.navigationBar.subviews containsObject:_partsControl]) {
        [self.navigationController.navigationBar addSubview:_partsControl];
    }
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [_partsControl removeFromSuperview];
}

#pragma mark - EGORefreshTableViewMethods
- (void)updateContent
{
    [super updateContent];
    if(_state == ScreenStateInfo) {
        [APIRequest requestPeopleInfoForId:_peopleID responseHandler:^(People *people) {
            [self contentUpdated];
            _currentPeople = people;
            dispatch_async(dispatch_get_main_queue(), ^{
                [_tableView reloadData];
            });
        }];
    }
    else {
        [APIRequest requestPeopleImagesForId:_peopleID responseHandler:^(NSArray *images) {
            [self contentUpdated];
            [_peopleImages removeAllObjects];
            [_peopleImages addObjectsFromArray:images];
            dispatch_async(dispatch_get_main_queue(), ^{
                [_tableView reloadData];
            });
        }];
    }
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (_state == ScreenStateInfo) {
        return InfoRowsCount;
    }
    else {
        NSInteger rows = (NSInteger)(_peopleImages.count / 3);
        if(_peopleImages.count % 3 != 0) {
            rows ++;
        }
        return rows;
    }
}

- (float)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (_state == ScreenStateInfo) {
        switch (indexPath.row) {
            case InfoRowMain: {
                return 60.0f;
            }
                break;
            case InfoRowStatus: {
                PeopleStatusCell *cell = (PeopleStatusCell *)[self tableView:tableView cellForRowAtIndexPath:indexPath];
                NSString *status = @"sadlkasjdlkasjdl ksaj dlkasjd lkasjd lkasjd lkasjd laksjd laksdj lsakdj laskdj laksd jaslk djsakl jdlka sjdl kajsd lkajsd lkasjd lkasjd laskd ladskj alsdkj laskdjaslkdjsalkd jaslkd jsalkd jdlk jlk jsdlk jkls djlkd jaldsk jlkadsjalkd jalsdk jalk djalksd jalsdk jdlkj alskd jalkdjaldk askld jaskd jalk djalsk djlkas dj";
                CGSize size = [status sizeWithFont:cell.peopleStatusLabel.font constrainedToSize:CGSizeMake(cell.peopleStatusLabel.frame.size.width, 1000)];
                return size.height + 31;
            }
                break;
            case InfoRowAdditional: {
                return 100.0f;
            }
                break;
            default: {
                NSAssert(NO, @"Unknown index path");
            }
                break;
        }
        return 0.0f;
    }
    else {
        return 110.0f;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if(_state == ScreenStateInfo) {
        return [self tableViewCellForPeopleInfoAtIndexPath:indexPath];;
    }
    else {
        return [self tableViewCellForImagesAtIndexPath:indexPath];
    }
}

- (UITableViewCell *)tableViewCellForPeopleInfoAtIndexPath:(NSIndexPath *)indexPath
{
    switch (indexPath.row) {
        case InfoRowMain: {
            static NSString * const reuseIdentifier = @"PeopleInfoMainReuseIdentifier";
            PeopleMainInfoCell *cell = [_tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
            if(!cell) {
                cell = [[PeopleMainInfoCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
                cell.selectionStyle = UITableViewCellSelectionStyleNone;
                cell.accessoryType = UITableViewCellAccessoryNone;
            }
            [cell setViewForPeople:_currentPeople];
            return cell;
        }
            break;
        case InfoRowStatus: {
            static NSString * const reuseIdentifier = @"PeopleInfoStatusReuseIdentifier";
            PeopleStatusCell *cell = [_tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
            if(!cell) {
                cell = [[PeopleStatusCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
                cell.selectionStyle = UITableViewCellSelectionStyleNone;
                cell.accessoryType = UITableViewCellAccessoryNone;
            }
            NSString *status = @"sadlkasjdlkasjdl ksaj dlkasjd lkasjd lkasjd lkasjd laksjd laksdj lsakdj laskdj laksd jaslk djsakl jdlka sjdl kajsd lkajsd lkasjd lkasjd laskd ladskj alsdkj laskdjaslkdjsalkd jaslkd jsalkd jdlk jlk jsdlk jkls djlkd jaldsk jlkadsjalkd jalsdk jalk djalksd jalsdk jdlkj alskd jalkdjaldk askld jaskd jalk djalsk djlkas dj";
            [cell setStatus:status];
            return cell;
        }
            break;
        case InfoRowAdditional: {
            static NSString * const reuseIdentifier = @"PeopleInfoAdditionalReuseIdentifier";
            PeopleInfoTableViewCell *cell = [_tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
            if(!cell) {
                cell = [[PeopleInfoTableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
                cell.selectionStyle = UITableViewCellSelectionStyleNone;
                cell.accessoryType = UITableViewCellAccessoryNone;
            }
            [cell setViewForPeople:_currentPeople];
            return cell;
        }
            break;
        default: {
            NSAssert(NO, @"Unknown index path");
        }
            break;
    }
    return nil;
}

- (UITableViewCell *)tableViewCellForImagesAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString * const reuseIdentifier = @"PeopleImagesReuseIdentifier";
    ImagesTableViewCell *cell = [_tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
    if(!cell) {
        cell = [[ImagesTableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.accessoryType = UITableViewCellAccessoryNone;
        cell.delegate = (id<ImagesTableViewCellDelegate>)self;
    }
    for(int i = 0; i < 3; i++) {
        NSInteger index = indexPath.row * 3 + i;
        [self loadImageForIndex:index atIndexPath:indexPath imagePosition:i];
    }
    return cell;
}

#pragma mark - UITableViewDelegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
}

#pragma mark - Image Loading Logic
- (void)loadImageForIndex:(NSInteger)index atIndexPath:(NSIndexPath *)indexPath imagePosition:(NSInteger)imagePosition
{
    if(index < _peopleImages.count) {
        Image *imageObject = [_peopleImages objectAtIndex:index];
        
        NSString *encodedURL = [imageObject.imageURL md5];
        NSString *path = [NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0];
        path = [path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.png",encodedURL]];
        if(![[NSFileManager defaultManager]fileExistsAtPath:path] && ![_loadingRequests containsObject:imageObject.imageURL]) {
            ImageDownloader *downloader = [[ImageDownloader alloc]initWithImageURL:[NSURL URLWithString:imageObject.imageURL]];
            downloader.successCallback = ^(ImageDownloader *downloader, UIImage *image){
                dispatch_async(dispatch_get_main_queue(), ^{
                    NSData *data = UIImagePNGRepresentation(image);
                    [data writeToFile:path atomically:YES];
                    if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                        UITableViewCell *cell = [_tableView cellForRowAtIndexPath:indexPath];
                        if([cell isKindOfClass:[ImagesTableViewCell class]]) {
                            ImagesTableViewCell *imageCell = (ImagesTableViewCell *)cell;
                            [imageCell setImage:image imagePosition:imagePosition];
                        }
                    }
                });
                [_loadingRequests removeObject:imageObject.imageURL];
            };
            downloader.failureCallback = ^(ImageDownloader *downloader, NSError *error){
                [_loadingRequests removeObject:imageObject.imageURL];
            };
            [downloader startDownload];
            [_loadingRequests addObject:imageObject.imageURL];
        }
        else {
            dispatch_async(dispatch_get_main_queue(), ^{
                if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                    UITableViewCell *cell = [_tableView cellForRowAtIndexPath:indexPath];
                    UIImage *savedImage = [UIImage imageWithContentsOfFile:path];
                    if([cell isKindOfClass:[ImagesTableViewCell class]]) {
                        ImagesTableViewCell *imageCell = (ImagesTableViewCell *)cell;
                        [imageCell setImage:savedImage imagePosition:imagePosition];
                    }
                }
            });
        }
    }
}

#pragma mark - ImagesTableViewCell delegae
- (void)didSelectImageOnPosition:(CellImagePosition)position inCell:(ImagesTableViewCell *)cell;
{
    NSIndexPath *indexPath = [_tableView indexPathForCell:cell];
    NSInteger index = indexPath.row * 3 + position;
    
    NSInteger currentUserId = [[[NSUserDefaults standardUserDefaults]objectForKey:kServerUserID]integerValue];
    if(currentUserId == _currentPeople.peopleID) {
        _currentImage = [_peopleImages objectAtIndex:index];
        NSString *title = NSLocalizedString(@"Выберите действие", @"Выберите действие");
        NSString *cancelTitle = NSLocalizedString(@"Отмена", @"Отмена");
        NSString *setAvatarTitle = NSLocalizedString(@"Установить на аватарку", @"Установить на аватарку");
        NSString *deleteTitle = NSLocalizedString(@"Удалить", @"Удалить");
        UIActionSheet *sheet = [[UIActionSheet alloc]initWithTitle:title delegate:self cancelButtonTitle:cancelTitle destructiveButtonTitle:nil otherButtonTitles:setAvatarTitle, deleteTitle, nil];
        [sheet showInView:self.view];
    }
    else {
        PeopleProfileViewController *controller = [[PeopleProfileViewController alloc]initWithPeople:_currentPeople startIndex:index];
        NavigationController *navController = [[NavigationController alloc]initWithRootViewController:controller];
        [self presentModalViewController:navController animated:YES];
    }
}

#pragma mark - ActionSheet delegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    switch (buttonIndex) {
        case 0: {
            //Set avatar action
            [APIRequest setAvatarRequestWithImageID:_currentImage.imageID];
        }
            break;
        case 1: {
            //delete action
            [APIRequest deletePhotoRequestWithImageID:_currentImage.imageID responseHandler:^{
                [self updateContent];
            }];
        }
            break;
        case 2: {
            //cancel action
        }
            break;
            
        default: {
            NSAssert(NO, @"Unknown button index in action sheet");
        }
            break;
    }
    _currentImage = nil;
}
@end