//
//  VKImagesImportControllerViewController.m
//  TeKtoTut
//
//  Created by Admin on 08.12.12.
//
//

#import "VKImagesImportControllerViewController.h"

#import "ImageDownloader.h"
#import "ImagesTableViewCell.h"
#import "URLRequest.h"
#import "APIRequest.h"

static NSInteger const kPhotosCount = 30;

@implementation VKImagesImportControllerViewController
{
    NSMutableArray *    _vkImages;
    NSCache *           _cache;
    NSMutableArray *    _loadingRequests;
    NSInteger           _imagesCount;
    NSMutableArray *    _selectedImages;
}


#pragma mark - Action
- (IBAction)nextPageButtonAction:(id)sender;
{
    [self loadNextPage];
}

- (IBAction)cancelButtonAction:(id)sender
{
    [self backButtonAction:nil];
}

- (IBAction)doneButtonAction:(id)sender
{
    [APIRequest uploadPhotoWithURLs:_selectedImages responseHandler:^{
        [self backButtonAction:nil];
    }];
}

#pragma mark - lifecycle
- (void)createBarButtons
{
    UIBarButtonItem *doneButton = [[UIBarButtonItem alloc]initWithTitle:NSLocalizedString(@"Готово", @"Готово") style:UIBarButtonItemStyleBordered target:self action:@selector(doneButtonAction:)];
    self.navigationItem.rightBarButtonItem = doneButton;
    
    UIBarButtonItem *cancelButton = [[UIBarButtonItem alloc]initWithTitle:NSLocalizedString(@"Отменить", @"Отменить") style:UIBarButtonItemStyleBordered target:self action:@selector(cancelButtonAction:)];
    self.navigationItem.leftBarButtonItem = cancelButton;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    _vkImages = [NSMutableArray array];
    _cache = [[NSCache alloc]init];
    _loadingRequests = [NSMutableArray array];
    _selectedImages = [NSMutableArray array];
    [self hideFooterViewIfNeed];
    [self updateContent];
    [self createBarButtons];
}

#pragma mark - VK Methods
- (void)hideFooterViewIfNeed
{
    if(![self isNextPageAvailable]) {
        _tableView.tableFooterView = nil;
    }
    else if(!_tableView.tableFooterView) {
        _tableView.tableFooterView = _tableFooterView;
    }
}

- (BOOL)isNextPageAvailable
{
    return (_vkImages.count < _imagesCount && _vkImages.count > 0);
}

- (void)loadNextPage
{
    NSString *accessToken = [[NSUserDefaults standardUserDefaults]objectForKey:kVKAccessTokenKey];
    NSString *requestString = [NSString stringWithFormat:@"https://api.vk.com/method/photos.getAll?no_service_albums=0&offset=%d&count=%d&extended=0&access_token=%@",_vkImages.count,kPhotosCount,accessToken];
    URLRequest *request = [URLRequest requestWithURL:[NSURL URLWithString:requestString] requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        NSDictionary *dictionary = [NSJSONSerialization JSONObjectWithData:response options:0 error:nil];
        NSArray *array = [dictionary objectForKey:@"response"];
        for(int i = 1; i < array.count; i++) {
            NSDictionary *imageDictionary = [array objectAtIndex:i];
            [_vkImages addObject:[imageDictionary objectForKey:@"src_big"]];
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            [super contentUpdated];
            [self hideFooterViewIfNeed];
            [_tableView reloadData];
        });
    }];
    [request startConnection];
}

- (void)loadVKImages
{
    NSString *accessToken = [[NSUserDefaults standardUserDefaults]objectForKey:kVKAccessTokenKey];
    NSString *requestString = [NSString stringWithFormat:@"https://api.vk.com/method/photos.getAll?no_service_albums=0&offset=0&count=%d&extended=0&access_token=%@",kPhotosCount,accessToken];
    URLRequest *request = [URLRequest requestWithURL:[NSURL URLWithString:requestString] requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        [_vkImages removeAllObjects];
        if(response) {
            NSDictionary *dictionary = [NSJSONSerialization JSONObjectWithData:response options:0 error:nil];
            NSArray *array = [dictionary objectForKey:@"response"];
            _imagesCount = [[array objectAtIndex:0]integerValue];
            for(int i = 1; i < array.count; i++) {
                NSDictionary *imageDictionary = [array objectAtIndex:i];
                [_vkImages addObject:[imageDictionary objectForKey:@"src_big"]];
            }
        }
        else {
            _imagesCount = 0;
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            [super contentUpdated];
            [self hideFooterViewIfNeed];
            [_tableView reloadData];
        });
    }];
    [request startConnection];
}

#pragma mark - EGO refresh
- (void)updateContent
{
    [super updateContent];
    [self loadVKImages];
}


#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSInteger rows = (NSInteger)(_vkImages.count / 3);
    if(_vkImages.count % 3 != 0) {
        rows ++;
    }
    return rows;
}

- (float)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 110.0f;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString * const reuseIdentifier = @"PeopleImagesReuseIdentifier";
    ImagesTableViewCell *cell = [_tableView dequeueReusableCellWithIdentifier:reuseIdentifier];
    if(!cell) {
        cell = [[ImagesTableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.accessoryType = UITableViewCellAccessoryNone;
        cell.delegate = (id<ImagesTableViewCellDelegate>)self;
    }
    for(int i = 0; i < 3; i++) {
        NSInteger index = indexPath.row * 3 + i;
        dispatch_sync(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
            [self loadImageForCell:cell index:index atIndexPath:indexPath onPosition:i];
        });
    }
    return cell;
}
#pragma mark - UITableViewDelegate

#pragma mark - Image Loading logic
- (void)loadImageForCell:(ImagesTableViewCell *)imageCell index:(NSInteger)index atIndexPath:(NSIndexPath *)indexPath onPosition:(NSInteger)imagePosition
{
    if(index < _vkImages.count) {
        NSString *imageURL = [_vkImages objectAtIndex:index];
        UIImage *cachedImage = [_cache objectForKey:imageURL];
        
        if(!cachedImage && ![_loadingRequests containsObject:imageURL]) {
            ImageDownloader *downloader = [[ImageDownloader alloc]initWithImageURL:[NSURL URLWithString:imageURL]];
            downloader.successCallback = ^(ImageDownloader *downloader, UIImage *image){
                [_cache setObject:image forKey:imageURL];
                dispatch_async(dispatch_get_main_queue(), ^{
                    if([_tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                        ImagesTableViewCell *loadedCell = (ImagesTableViewCell *)[_tableView cellForRowAtIndexPath:indexPath];
                        [loadedCell setImage:image imagePosition:imagePosition];
                        if([_selectedImages containsObject:imageURL]) {
                            [loadedCell setImageViewSelected:YES imageViewPosition:imagePosition];
                        }
                    }
                });
                [_loadingRequests removeObject:imageURL];
            };
            downloader.failureCallback = ^(ImageDownloader *downloader, NSError *error){
                [_loadingRequests removeObject:imageURL];
            };
            [downloader startDownload];
            [_loadingRequests addObject:imageURL];
        }
        else {
            dispatch_async(dispatch_get_main_queue(), ^{
                [imageCell setImage:cachedImage imagePosition:imagePosition];
                if([_selectedImages containsObject:imageURL]) {
                    [imageCell setImageViewSelected:YES imageViewPosition:imagePosition];
                }
            });
        }
    }
}

#pragma mark - ImagesTableViewCell delegae
- (void)didSelectImageOnPosition:(CellImagePosition)position inCell:(ImagesTableViewCell *)cell;
{
    NSIndexPath *indexPath = [_tableView indexPathForCell:cell];
    NSInteger index = indexPath.row * 3 + position;
    NSString *imageURL = [_vkImages objectAtIndex:index];
    if([_selectedImages containsObject:imageURL]) {
        [cell setImageViewSelected:NO imageViewPosition:position];
        [_selectedImages removeObject:imageURL];
    }
    else {
        [cell setImageViewSelected:YES imageViewPosition:position];
        [_selectedImages addObject:imageURL];
    }
}
@end
