//
//  VKWebViewController.m
//  TeKtoTut
//
//  Created by mave on 12/6/12.
//
//

#import "VKWebViewController.h"
#import "ServerRequest.h"
#import "APIRequest.h"

#define RedirectURL @"https://oauth.vk.com/blank.html"
#define APP_ID @"3321415"

static NSString * const kAccessTokenKey = @"access_token";
static NSString * const kUserIDKey      = @"user_id";
static NSString * const kExpiresInKey   = @"expires_in";

@implementation VKWebViewController

#pragma mark - class methods
+ (id)authRequestController
{
    NSURL *url = [NSURL URLWithString:[NSString stringWithFormat:@"https://oauth.vk.com/authorize?client_id=%@&scope=notify,friends,photos,status,wall,offline&redirect_uri=%@&response_type=token&display=touch",APP_ID,RedirectURL]];
    return [[[self class]alloc]initWithURL:url];
}

#pragma mark - Actions
- (IBAction)cancelButtonAction:(id)sender
{
    if([_delegate respondsToSelector:@selector(didCancelLogin)]) {
        [_delegate didCancelLogin];
    }
}

#pragma mark - lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    [self createBarButtons];
}

#pragma mark - working methods
- (void)createBarButtons
{
    UIBarButtonItem *cancelButton = [[UIBarButtonItem alloc]initWithTitle:NSLocalizedString(@"Cancel", @"Cancel") style:UIBarButtonItemStyleBordered target:self action:@selector(cancelButtonAction:)];
    self.navigationItem.leftBarButtonItem = cancelButton;
}

- (NSDictionary *)parseVKResponse:(NSString *)response
{
    NSRange range = [response rangeOfString:@"#"];
    if(range.location != NSNotFound) {
        NSMutableDictionary *dictionary = [NSMutableDictionary dictionary];
        NSString *parameterString = [response substringFromIndex:(range.location + 1)];
        NSArray *parameterArray = [parameterString componentsSeparatedByString:@"&"];
        for(NSString *param in parameterArray) {
            NSArray *parameters = [param componentsSeparatedByString:@"="];
            if(parameters.count > 1) {
                NSString *parName = [parameters objectAtIndex:0];
                NSString *parValue = [parameters objectAtIndex:1];
                [dictionary setObject:parValue forKey:parName];
            }
        }
        return dictionary;
    }
    else {
        return nil;
    }
}

#pragma mark - UIWebViewDelegate
- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType
{
    NSRange range = [request.URL.absoluteString rangeOfString:RedirectURL];
    if(range.location == 0) {
        NSLog(@"%@",request.URL.absoluteString);
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
            NSString *errorTitle = NSLocalizedString(@"Не удалось войти", @"Не удалось войти");
            NSString *errorMessage = NSLocalizedString(@"Приложение не было авторизовано", @"Приложение не было авторизовано");
            NSDictionary *responseParams = [self parseVKResponse:request.URL.absoluteString];
            if(responseParams) {
                if([_delegate respondsToSelector:@selector(didLoginWithVKID:vkToken:)]) {
                    [_delegate didLoginWithVKID:[[responseParams objectForKey:kUserIDKey]integerValue] vkToken:[responseParams objectForKey:kAccessTokenKey]];
                }
            }
            else {
                UIAlertView *alertView = [[UIAlertView alloc]initWithTitle:errorTitle message:errorMessage delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
                [alertView show];
            }
            
        });
        return NO;
    }
    else {
        return YES;
    }
}

#pragma mark - UIAlertView delegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    [self cancelButtonAction:nil];
}

@end
