//
//  OOWebBrowserViewController.m
//  OpenOcean
//
//  Created by Alexander Chernyshev on 06/09/2012.
//  Copyright (c) 2012 Sibers. All rights reserved.
//

#import "WebBrowserViewController.h"

#import <QuartzCore/QuartzCore.h>

#import "AppDelegate.h"

typedef enum
{
    WebViewStateLoading,
    WebViewStateNormal
}WebViewState;

@implementation WebBrowserViewController
{
    WebViewState _state;
}
@synthesize webView = _webView;

#pragma mark - Actions

- (IBAction)backWebButtonAction:(id)sender
{
    if([_webView canGoBack]) {
        [_webView goBack];
    }
    else {
        [self backButtonAction:nil];
    }
}

- (IBAction)forwardWebButtonAction:(id)sender
{
    if([_webView canGoForward]) {
        [_webView goForward];
    }
}

- (IBAction)updateWebButtonAction:(id)sender
{
    if(_state == WebViewStateLoading) {
        [_webView stopLoading];
    }
    else {
        [_webView reload];
    }
}

#pragma mark - initialization
- (void)createWebView
{
    float offset;
    //if(self.navigationController && !self.navigationController.navigationBarHidden) {
        offset = 88;
    /*}
    else {
        offset = 44;
    }*/
    CGRect frame = CGRectMake(0, 0, self.view.bounds.size.width, self.view.bounds.size.height - offset);
    _webView = [[UIWebView alloc]initWithFrame:frame];
    _webView.delegate = self;
    _webView.contentMode = UIViewContentModeScaleAspectFit;
    [self.view addSubview:_webView];
    
    _browserControllsBar = [[UIToolbar alloc]initWithFrame:CGRectMake(0, self.view.bounds.size.height - offset, self.view.bounds.size.width, 44)];
    [self.view addSubview:_browserControllsBar];
    
    _backButton = [[UIBarButtonItem alloc]initWithImage:[UIImage imageNamed:@"browser_back.png"] style:UIBarButtonItemStyleBordered target:self action:@selector(backWebButtonAction:)];
    _updateButton = [[UIBarButtonItem alloc]initWithImage:[UIImage imageNamed:@"browser_refresh.png"] style:UIBarButtonItemStyleBordered target:self action:@selector(updateWebButtonAction:)];
    _forwardButton = [[UIBarButtonItem alloc]initWithImage:[UIImage imageNamed:@"browser_next.png"] style:UIBarButtonItemStyleBordered target:self action:@selector(forwardWebButtonAction:)];
    
    UIBarButtonItem *space = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    UIBarButtonItem *space1 = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    
    [_browserControllsBar setItems:[NSArray arrayWithObjects:space, _backButton, _updateButton, _forwardButton, space1, nil]];
}

- (id)initWithURL:(NSURL *)url
{
    self = [super init];
    if(self) {
        [self createWebView];
        [_webView loadRequest:[NSURLRequest requestWithURL:url]];
    }
    return self;
}

- (id)initWithHTMLString:(NSString *)htmlString
{
    self = [super init];
    if(self) {
        [self createWebView];
        [_webView loadHTMLString:htmlString baseURL:nil];
    }
    return self;
}

#pragma mark - lifecycle
- (void)createDownloadingView
{
    _downloadingView = [[UIView alloc]initWithFrame:CGRectMake(0, 0, 100, 45)];
    _downloadingView.backgroundColor = [UIColor blackColor];
    _downloadingView.alpha = 0.8;
    _downloadingView.layer.cornerRadius = 10.0;
    _downloadingView.layer.masksToBounds = YES;
    
    
    UIActivityIndicatorView *actView = [[UIActivityIndicatorView alloc]initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhite];
    [actView startAnimating];
    actView.center = CGPointMake(50, 15);
    [_downloadingView addSubview:actView];
    
    UILabel *label = [UILabel new];
    label.backgroundColor = [UIColor clearColor];
    label.frame = CGRectMake(10, 25, 80, 20);
    label.numberOfLines = 1;
    label.font = [UIFont boldSystemFontOfSize:15.0];
    label.text = @"Loading...";
    label.textColor = [UIColor whiteColor];
    label.textAlignment = NSTextAlignmentCenter;
    [_downloadingView addSubview:label];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [self createDownloadingView];
}

- (void)viewDidUnload
{
    _webView.delegate = nil;
    _webView = nil;
    _backButton = nil;
    _forwardButton = nil;
    _updateButton = nil;
    _downloadingView = nil;
    [super viewDidUnload];
}

- (void)dealloc
{
    _webView.delegate = nil;
    _webView = nil;
    _backButton = nil;
    _forwardButton = nil;
    _updateButton = nil;
    _downloadingView = nil;
}

#pragma mark - working methods
- (void)setBrowserControllsHidden:(BOOL)hidden
{
    CGRect frame;
    if(hidden) {
        frame = CGRectMake(_webView.frame.origin.x, _webView.frame.origin.y, _webView.frame.size.width, _webView.frame.size.height + _browserControllsBar.frame.size.height);
    }
    else {
        frame = CGRectMake(_webView.frame.origin.x, _webView.frame.origin.y, _webView.frame.size.width, _webView.frame.size.height - _browserControllsBar.frame.size.height);
    }
    [_browserControllsBar setHidden:hidden];
    _webView.frame = frame;
}

- (void)showDownloadingView
{
    [self.view addSubview:_downloadingView];
    _downloadingView.center = self.view.center;
}

- (void)hideDownloadingView
{
    [_downloadingView removeFromSuperview];
}

#pragma mark - UIWebViewDelegate methods
- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType
{
    return YES;
}
- (void)webViewDidStartLoad:(UIWebView *)webView
{
    _state = WebViewStateLoading;
    _updateButton.image = [UIImage imageNamed:@"browser_x.png"];
    
    if([_webView canGoForward]) {
        _forwardButton.enabled = YES;
    }
    else {
        _forwardButton.enabled = NO;
    }
    
    [self showDownloadingView];
}

- (void)webViewDidFinishLoad:(UIWebView *)webView
{
    [self hideDownloadingView];
    
    _state = WebViewStateNormal;
    _updateButton.image = [UIImage imageNamed:@"browser_refresh.png"];
    
    if([_webView canGoForward]) {
        _forwardButton.enabled = YES;
    }
    else {
        _forwardButton.enabled = NO;
    }
}

- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error
{    
    [self hideDownloadingView];
    
    _state = WebViewStateNormal;
    _updateButton.image = [UIImage imageNamed:@"browser_refresh.png"];
    
    if([_webView canGoForward]) {
        _forwardButton.enabled = YES;
    }
    else {
        _forwardButton.enabled = NO;
    }

}
@end