//
//  APIRequest.m
//  TeKtoTut
//
//  Created by Admin on 01.12.12.
//
//

#import "APIRequest.h"

#import "Place.h"
#import "ServerRequest.h"
#import "People.h"
#import "Image.h"
#import "PeoplesList.h"

@implementation APIRequest

+ (NSString *)signedRequest:(NSString *)requestString withParams:(BOOL)hasParams
{
    NSString *serverToken = [[NSUserDefaults standardUserDefaults]objectForKey:kServerAccessTokenKey];
    if(serverToken) {
        if(hasParams) {
            NSString *signedString = [requestString stringByAppendingFormat:@"&access_token=%@",serverToken];
            return signedString;
        }
        else {
            NSString *signedString = [requestString stringByAppendingFormat:@"?access_token=%@",serverToken];
            return signedString;
        }
    }
    else {
        return requestString;
    }
}

+ (void)handleServerError:(NSError *)error
{
    NSLog(@"%@",error);
    switch (error.code) {
        case kUserNotLoginErrorCode: {
            dispatch_async(dispatch_get_main_queue(), ^{
                [[NSNotificationCenter defaultCenter] postNotificationName:kNeedLoginNotification object:nil];
            });
        }
            break;
        case kPreprocessErrorCode: {
            
        }
            break;
        case kProfileInfoMissingErrorCode: {
            
        }
            break;
            
        default: {
            
        }
            break;
    }
}

+ (void)requestPeopleForPlace:(Place *)place responseHandler:(void (^)(NSArray *))handler
{
    NSString *requestString = [NSString stringWithFormat:@"%@?pid=%d",kPlacePeoplesRequestURL, place.placeID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        NSMutableArray *array = [NSMutableArray array];
        if(!error) {
            NSArray *peoples = [(NSDictionary *)response objectForKey:@"response"];
            for(NSDictionary *dictionary in peoples) {
                People *people = [[People alloc]initWithDictionary:dictionary];
                [array addObject:people];
            }
        }
        else {
            [self handleServerError:error];
        }
        handler(array);
    }];
    [request startConnection];
}

+ (void)requestPeopleInfoForId:(NSInteger)peopleID responseHandler:(void (^)(People *))handler
{
    NSString *requestString = [NSString stringWithFormat:@"%@?uid=%d",kGetPeopleInfoRequestURL,peopleID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        People *people = nil;
        if(!error) {
            people = [[People alloc]initWithDictionary:[response objectForKey:@"response"]];
        }
        else {
            [self handleServerError:error];
        }
        handler(people);
    }];
    [request startConnection];
}

+ (void)requestPeopleImagesForId:(NSInteger)peopleID responseHandler:(void (^)(NSArray *images))handler
{
    NSString *requestString = [NSString stringWithFormat:@"%@?uid=%d",kGetPeopleImagesRequestURL,peopleID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        NSMutableArray *array = [NSMutableArray array];
        if(!error) {
            NSArray *images = [response objectForKey:@"response"];
            for(NSDictionary *dictionary in images) {
                Image *image = [[Image alloc]initWithDictionary:dictionary];
                [array addObject:image];
            }
        }
        else {
            [self handleServerError:error];
        }
        handler(array);
    }];
    [request startConnection];
}

+ (void)setAvatarRequestWithImageID:(NSInteger)imageID
{
    NSString *requestString = [NSString stringWithFormat:@"%@?id=%d",kSetAvatarRequestURL,imageID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        if(error) {
            [self handleServerError:error];
        }
        else {
            NSLog(@"%@",response);
        }
    }];
    [request startConnection];
}

+ (void)deletePhotoRequestWithImageID:(NSInteger)imageID responseHandler:(dispatch_block_t)handler;
{
    NSString *requestString = [NSString stringWithFormat:@"%@?id=%d",kDeletePhotoRequestURL,imageID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        if(error) {
            [self handleServerError:error];
        }
        handler();
    }];
    [request startConnection];
}

+ (void)uploadPhotoWithURLs:(NSArray *)array responseHandler:(dispatch_block_t)handler
{
    if(array.count) {
        NSString *urls = @"";
        for(int i = 0; i < (array.count - 1); i++) {
            NSString *url = [array objectAtIndex:i];
            urls = [urls stringByAppendingString:[NSString stringWithFormat:@"%@,",url]];
        }
        NSString *url = [array lastObject];
        urls = [urls stringByAppendingString:url];
        NSString *requestString = [NSString stringWithFormat:@"%@?url=%@",kUploadPhotoRequestURL,urls];
        requestString = [self signedRequest:requestString withParams:YES];
        ServerRequest *request = [ServerRequest requestWithURL:[NSURL URLWithString:requestString] requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
            if(error) {
                [self handleServerError:error];
            }
            handler();
        }];
        [request startConnection];
    }
    else {
        handler();
    }
}

+ (void)updatePlaceInfoForPlace:(Place *)place responseHandler:(dispatch_block_t)handler;
{
    NSString *requestString = [NSString stringWithFormat:@"%@?pid=%d",kPlaceUpdateInfoRequestURL,place.placeID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        if(!error) {
            NSDictionary *dictionary = [(NSDictionary *)response objectForKey:@"response"];
            [place updateWithDictionary:dictionary];
        }
        else {
            [self handleServerError:error];
        }
        handler();
    }];
    [request startConnection];
}

+ (void)sendMessageWithSenderID:(NSInteger)senderID messageText:(NSString *)text responseHandler:(void (^)(BOOL success)) handler
{
    NSString *escapedText = (NSString *)CFBridgingRelease(CFURLCreateStringByAddingPercentEscapes(kCFAllocatorDefault, (CFStringRef)text, NULL, CFSTR(":/?#[]@!$&’()*+,;='"), kCFStringEncodingUTF8));
    if(escapedText && ![escapedText isEqualToString:@""]) {
        NSString *requestString = [NSString stringWithFormat:@"%@?uid=%d&message=%@",kMessageSendURL,senderID, escapedText];
        requestString = [self signedRequest:requestString withParams:YES];
        NSURL *url = [NSURL URLWithString:requestString];
        ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
            if(!error) {
                handler(YES);
            }
            else {
                [self handleServerError:error];
                handler(NO);
            }
        }];
        [request startConnection];
    }
}

+ (void)checkinRequestForPlace:(Place *)place responseHandler:(dispatch_block_t)handler;
{
    NSString *requestString = [NSString stringWithFormat:@"%@?pid=%d",kPlaceCheckinRequest,place.placeID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        handler();
        if(!error) {
            BOOL success = [[response objectForKey:@"response"]boolValue];
            if(success) {
                BOOL isWallPostDisable = [[[NSUserDefaults standardUserDefaults] objectForKey:kWallPostEnableKey] boolValue];
                if(!isWallPostDisable) {
                    NSString *accessToken = [[NSUserDefaults standardUserDefaults] objectForKey:kVKAccessTokenKey];
                    NSString *profileRequestString = [NSString stringWithFormat:@"https://api.vk.com/method/getProfiles?access_token=%@",accessToken];
                    URLRequest *profileRequest = [URLRequest requestWithURL:[NSURL URLWithString:profileRequestString] requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
                        if(!error) {
                            NSDictionary *responseDict1 = [NSJSONSerialization JSONObjectWithData:response options:0 error:nil];
                            NSArray *array = [responseDict1 objectForKey:@"response"];
                            NSString *userName = [[array objectAtIndex:0] objectForKey:@"first_name"];
                            
                            
                            NSString *placeURL = [NSString stringWithFormat:@"http://www.tektotut.ru/places/%d",place.placeID];
                            NSString *message = [NSString stringWithFormat:@"%@ сейчас в %@. Приходи к нам!",userName,place.name];
                            NSString *escapedText = (NSString *)CFBridgingRelease(CFURLCreateStringByAddingPercentEscapes(kCFAllocatorDefault, (CFStringRef)message, NULL, CFSTR(":/?#[]@!$&’()*+,;='"), kCFStringEncodingUTF8));
                            NSString *wallRequestString = [NSString stringWithFormat:@"https://api.vk.com/method/wall.post?message=%@&attachments=%@&access_token=%@",escapedText,placeURL,accessToken];
                            URLRequest *wallRequest = [URLRequest requestWithURL:[NSURL URLWithString:wallRequestString] requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
                                NSDictionary *dict = [NSJSONSerialization JSONObjectWithData:response options:0 error:nil];
                                NSLog(@"%@",error);
                                NSLog(@"%@",dict);
                            }];
                            [wallRequest startConnection];
                        }
                    }];
                    [profileRequest startConnection];
                }
            }
        }
        else {
            [self handleServerError:error];
        }
    }];
    [request startConnection];
}

+ (void)savePlaceGuests:(NSInteger)placeID
{
    NSString *requestString = [self signedRequest:kSavePlaceGuestsURL withParams:NO];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        if(!error) {
            NSLog(@"%@",response);
        }
        else {
            NSLog(@"%@",error);
            [self handleServerError:error];
        }
    }];
    [request startConnection];
}

+ (void)loadSavedGuestsWithResponseHandler:(void (^)(NSArray *guests))handler
{
    NSString *requestString = [self signedRequest:kLoadSavedGuestsURL withParams:NO];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        NSMutableArray *resultArray = [NSMutableArray array];
        if(!error) {
            NSArray *array = [response objectForKey:@"response"];
            for(NSDictionary *dictionary in array) {
                PeoplesList *list = [[PeoplesList alloc]initWithDictionary:dictionary];
                [resultArray addObject:list];
            }
        }
        else {
            NSLog(@"%@",error);
            [self handleServerError:error];
        }
        handler(resultArray);
    }];
    [request startConnection];
}

+ (void)loadPlacePeoplesWithID:(NSInteger)listID responseHandler:(void(^)(NSArray *peoples))handler
{
    NSString *requestString = [NSString stringWithFormat:@"%@?pgid=%d",kLoadPlacePeoplesURL,listID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        if(!error) {
            NSLog(@"%@",response);
            NSArray *array = [response objectForKey:@"response"];
            NSMutableArray *resultArray = [NSMutableArray arrayWithCapacity:array.count];
            for(NSDictionary *dictionary in array) {
                NSDictionary *peopleDictionary = [dictionary objectForKey:@"user"];
                People *list = [[People alloc]initWithDictionary:peopleDictionary];
                [resultArray addObject:list];
            }
            handler(resultArray);
        }
        else {
            NSLog(@"%@",error);
            [self handleServerError:error];
            handler(nil);
        }
    }];
    [request startConnection];
}

+ (void)updatePeopleInfo:(People *)peopleInfo
{
    //TODO: implement this method for real
    NSString *requestString = [NSString stringWithFormat:@"%@?first_name=Alexandr&city=Novosibirs&bdate=2182135",kUpdatePeopleInfoURL];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        if(!error) {
        }
        else {
            NSLog(@"%@",error);
        }
    }];
    [request startConnection];
}

+ (void)markAsReadMessageWithID:(NSInteger)messageID responseHandler:(void (^)(BOOL success)) handler;
{
    NSString *requestString = [NSString stringWithFormat:@"%@?mids=%d",kMarkMessageAsReadURL, messageID];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:^(id response, NSError *error) {
        if(!error) {
            BOOL flag = [[response objectForKey:@"response"] boolValue];
            handler(flag);
            NSLog(@"%@",response);
        }
        else {
            [self handleServerError:error];
            handler(NO);
        }
    }];
    [request startConnection];
}

+ (void)updatePushNotificationToken:(NSString *)tokenString
{
    NSString *requestString = [NSString stringWithFormat:@"%@",kUpdatePushTokenURL];
    requestString = [self signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url requestMethod:RequestMethodGet responseHandler:nil];
    [request startConnection];
}
@end