//
//  ImageDownloadQueue.h
//  engagis
//
//  Created by Egor Chiglintsev on 7/23/12.
//  Copyright (c) 2012 Sibers. All rights reserved.
//

#pragma mark -
#pragma mark ImageDownloadQueue interface

static NSString * const kNotificationImageDownloadUpdated = @"NotificationImageDownloadUpdated";
static NSString * const kNotificationImageDownloadEnd     = @"NotificationImageDownloadEnd";


@interface ImageDownloadQueue : NSObject
{
    NSMutableArray *_queuedImageURLs;    
    NSMutableArray *_downloaders;
}


+ (id) shared NS_RETURNS_NOT_RETAINED;

- (CGFloat) downloadProgress;

/* Cancels current download and clears the queue */
- (void) clearQueue; 


/*
    Stores all remaining image URLs to user defaults and
    clears the queue.
*/
- (void) storeAllRemainingURLsAndCancel;


/*
    Queues the image URLs remaining from the previous session.
 
    When app first receives the list of property, all property
    photos are queued to be downloaded at once. Downloading all
    of these may take some significant time so it is most likely
    that the user would close the app before all of the photos 
    from the queue are successfully downloaded. URLs for photos
    not yet downloaded are stored in the user defaults and this
    method requeues all photos from this 'not yet downloaded' 
    list.
*/
- (void) resumeDownload;



/* Adds a photo URL to the queue */
- (void) queuePhotoWithURL: (NSString *) URLString;

- (BOOL)hasImageForDownload;
- (void)startDownloading;
- (void)stopDownloading;
@end