//
//  ImageDownloader.h
//  SingaporeVenues
//
//  Created by Egor Chiglintsev on 12/12/11.
//  Copyright (c) 2011 Sibers. All rights reserved.
//

#import <Foundation/Foundation.h>

@class ImageDownloader;

#pragma mark -
#pragma mark Constants

//DECLARE_ERROR_DOMAIN(ImageDownloaderErrorDomain);

enum
{
    ImageDownloaderErrorCouldNotDecodeImage = 1
} ImageDownloaderError;


#pragma mark -
#pragma mark Typedefs

typedef void (^ImageDownloaderSuccessCallback)(ImageDownloader *downloader, UIImage *image);
typedef void (^ImageDownloaderFailureCallback)(ImageDownloader *downloader, NSError *error);

#pragma mark -
#pragma mark ImageDownloader interface


/*!
    This class is used to download images from the web asyncronously
    thus not blocking the main thread of the application.
 */
@interface ImageDownloader : NSObject
{
    NSMutableData   *_downloadedData;
    NSURLConnection *_connection; 
    UIImage         *_image;
    NSURL *          _downloadedURL;
    
    ImageDownloaderSuccessCallback _successCallback;
    ImageDownloaderFailureCallback _failureCallback;
}

@property (nonatomic, copy) ImageDownloaderSuccessCallback successCallback;
@property (nonatomic, copy) ImageDownloaderFailureCallback failureCallback;

- (id)   initWithImageURL: (NSURL *) imageUrl;
- (void) startDownload;
- (void) cancelDownload;
- (void) restartDownload;

@end
