//
//  ImageDownloader.m
//  SingaporeVenues
//
//  Created by Egor Chiglintsev on 12/12/11.
//  Copyright (c) 2011 Sibers. All rights reserved.
//

#if (!__has_feature(objc_arc))
#error "This file should be compiled with ARC support"
#endif

#import "ImageDownloader.h"


#pragma mark -
#pragma mark Constants

//DEFINE_ERROR_DOMAIN(ImageDownloaderErrorDomain)


#pragma mark -
#pragma mark ImageDownloader private

@interface ImageDownloader()

- (void) didDownloadImage: (UIImage *) image;
- (void) didFailWithError: (NSError *) error;

@end


#pragma mark -
#pragma mark ImageDownloader implementation

@implementation ImageDownloader


#pragma mark -
#pragma mark Properties

@synthesize successCallback = _successCallback;
@synthesize failureCallback = _failureCallback;


#pragma mark -
#pragma mark initialization methods

- (id) initWithImageURL: (NSURL *) imageUrl
{
    self = [super init];
    
    if (self != nil)
    {
        _downloadedURL = imageUrl;
        NSURLRequest *request = [[NSURLRequest alloc] initWithURL: imageUrl];
        _connection           = [[NSURLConnection alloc] initWithRequest: request 
                                                                delegate: self];
        _downloadedData       = [NSMutableData data];
    }
    
    return self;
}


#pragma mark -
#pragma mark Methods

- (void) startDownload
{
    [_connection start];
}


- (void) cancelDownload
{
    [_connection cancel];
}

- (void)restartDownload
{
    [self cancelDownload];
    _downloadedData = nil;
    _connection     = nil;
    NSURLRequest *request = [[NSURLRequest alloc] initWithURL: _downloadedURL];
    _connection           = [[NSURLConnection alloc] initWithRequest: request
                                                            delegate: self];
    _downloadedData       = [NSMutableData data];
    [self startDownload];
}


#pragma mark -
#pragma mark NSURLConnection delegate

- (void) connection: (NSURLConnection *) connection 
     didReceiveData: (NSData *)          data
{
    [_downloadedData appendData: data];
}


- (void) connection: (NSURLConnection *) connection 
   didFailWithError: (NSError *)         error
{
    _downloadedData = nil;
    _connection     = nil;
    
    [self didFailWithError: error];
}


- (void) connectionDidFinishLoading: (NSURLConnection *) connection
{
    _image          = [[UIImage alloc] initWithData: _downloadedData];
    _downloadedData = nil;
    _connection     = nil;
        
    if (_image != nil)
    {
        [self didDownloadImage: _image];
    }
    else
    {
        NSError *error = [[NSError alloc] initWithDomain: @"ImageDownloaderErrorDomain"
                                                    code: ImageDownloaderErrorCouldNotDecodeImage
                                                userInfo: nil];
        
        [self didFailWithError: error];
    }
}


#pragma mark -
#pragma mark Callbacks

- (void) didDownloadImage: (UIImage *) image
{
    if (_successCallback != nil)
    {
        _successCallback(self, image);
    }
}


- (void) didFailWithError: (NSError *) error
{
    if (_failureCallback != nil)
    {
        _failureCallback(self, error);
    }
}

@end
