//
//  ImageLoader.m
//  TeKtoTut
//
//  Created by Admin on 26/01/2013.
//
//

#import "ImageLoader.h"

#import "ImageDownloader.h"
#import "NSString+MD5.h"

@implementation ImageLoader

- (id)initWithDelegate:(id<ImageLoaderDelegate>)delegate
{
    self = [super init];
    if(self) {
        _loadingURLs = [NSMutableDictionary dictionary];
        self.delegate = delegate;
    }
    return self;
}

- (void)loadImageForURL:(NSString *)loadingURL atIndexPath:(NSIndexPath *)indexPath
{
    NSString *imagePath = [self pathForImageAtURL:loadingURL];
    if([self hasStoredImageAtPath:imagePath]) {
        if([_delegate respondsToSelector:@selector(didLoadImage:forPath:)]) {
            UIImage *image = [UIImage imageWithContentsOfFile:imagePath];
            NSArray *indexesArray = [NSArray arrayWithObject:indexPath];
            [_delegate didLoadImage:image forPath:indexesArray];
        }
    }
    else {
        NSMutableArray *pathsArray = [_loadingURLs objectForKey:loadingURL];
        if(!pathsArray) {
            pathsArray = [NSMutableArray array];
            [pathsArray addObject:indexPath];
            [_loadingURLs setObject:pathsArray forKey:loadingURL];
            
            ImageDownloader *downloader = [[ImageDownloader alloc]initWithImageURL:[NSURL URLWithString:loadingURL]];
            downloader.successCallback = ^(ImageDownloader *downloader, UIImage *image){
                NSData *data = UIImagePNGRepresentation(image);
                [data writeToFile:imagePath atomically:YES];
                if([_delegate respondsToSelector:@selector(didLoadImage:forPath:)]) {
                    NSArray *indexesArray = [_loadingURLs objectForKey:loadingURL];
                    [_delegate didLoadImage:image forPath:indexesArray];
                }
                [_loadingURLs removeObjectForKey:loadingURL];
            };
            downloader.failureCallback = ^(ImageDownloader *downloader, NSError *error){
                [_loadingURLs removeObjectForKey:loadingURL];
            };
            [downloader startDownload];
        }
        else {
            [pathsArray addObject:indexPath];
            [_loadingURLs setObject:pathsArray forKey:loadingURL];
        }
        if([_delegate respondsToSelector:@selector(didFailLoadingImageForPath:)]) {
            [_delegate didFailLoadingImageForPath:pathsArray];
        }
    }
}

- (NSString *)pathForImageAtURL:(NSString *)urlString
{
    NSString *encodedURL = [urlString md5];
    NSString *path = [NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path = [path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.png",encodedURL]];
    return path;
}

- (BOOL)hasStoredImageAtPath:(NSString *)path
{
    return [[NSFileManager defaultManager] fileExistsAtPath:path];
}
@end
