//
//  OOLocationManager.m
//  OpenOcean
//
//  Created by Egor Chiglintsev on 7/2/12.
//  Copyright (c) 2012 Sibers. All rights reserved.
//

#if (!__has_feature(objc_arc))
#error "This file should be compiled with ARC support"
#endif

#import "LocationManager.h"

@implementation LocationManager

#pragma mark -
#pragma mark Singleton

+ (id) shared
{
    static dispatch_once_t predicate  = 0;
    __strong static id sharedInstance = nil;
    
    dispatch_once(&predicate, 
                  ^{
                      sharedInstance = [[self alloc] init];
                  });
    return sharedInstance;

}

#pragma mark -
#pragma mark Class Methods
+ (BOOL) locationServicesEnabled
{
    CLAuthorizationStatus status = [CLLocationManager authorizationStatus];
    return [CLLocationManager locationServicesEnabled] && status == kCLAuthorizationStatusAuthorized;
}

+ (void) startUpdatingLocation
{
    [[self shared] startUpdatingLocation];
}

+ (void) stopUpdatingLocation
{
    [[self shared] stopUpdatingLocation];
}

+ (CLLocation *) currentLocation
{
    return ((LocationManager *)[self shared])->_currentLocation;
}

#pragma mark -
#pragma mark Instance methods

- (void) startUpdatingLocation
{
    if (_locationManager == nil)
    {
        _locationManager = [CLLocationManager new];
        _locationManager.desiredAccuracy = kCLLocationAccuracyHundredMeters;
        _locationManager.delegate = self;
    }
    
    [_locationManager startUpdatingLocation];
}

- (void) stopUpdatingLocation
{
    [_locationManager stopUpdatingLocation];
}

#pragma mark -
#pragma mark CLLocationManagerDelegate

- (void) locationManager: (CLLocationManager *) manager 
     didUpdateToLocation: (CLLocation *) newLocation 
            fromLocation: (CLLocation *) oldLocation
{
    _currentLocation = newLocation;
    [[NSNotificationCenter defaultCenter]postNotificationName:kUserLocationUpdated object:nil];
}


- (void) locationManager: (CLLocationManager *) manager 
        didFailWithError: (NSError *) error
{
    if ([error.domain isEqualToString: kCLErrorDomain])
    {
        switch (error.code)
        {
            case kCLErrorDenied:
            {
                /*
                    The doc states we should stop the location service
                    if we receive a kCLErrorDenied.
                 */
                [self stopUpdatingLocation];
            } break;
                
            default:
            {
                // Ignore everything else
            } break;
        }
    }
    [[NSNotificationCenter defaultCenter]postNotificationName:kUserLocationUpdated object:nil];
}

@end
