//
//  ObjectsList.m
//  TeKtoTut
//
//  Created by mave on 1/10/13.
//
//

#import "ObjectsList.h"

#import "ServerRequest.h"
#import "APIRequest.h"

@implementation ObjectsList

#pragma mark - initialization
- (id)initWithAPIMethod:(NSString *)method
           objectsCount:(NSInteger)count
{
    self = [super init];
    if(self) {
        self.loadedObjects = [NSMutableArray array];
        _objectsCount = 0;
        _APIMethod = method;
        _objectsInRequest = count;
    }
    return self;
}

- (void)loadObjectsForNextPageWithParameters:(NSString *)parameters
{
    if([self isNextObjectsPageAvailable]) {
        NSString *requestString = _APIMethod;
        if(parameters) {
            requestString = [requestString stringByAppendingFormat:@"?%@&count=%d&offset=%d",parameters, _objectsInRequest, self.loadedObjects.count];
        }
        else {
            requestString = [requestString stringByAppendingFormat:@"?count=%d&offset=%d", _objectsInRequest, self.loadedObjects.count];
        }
        requestString = [APIRequest signedRequest:requestString withParams:YES];
        NSURL *url = [NSURL URLWithString:requestString];
        ServerRequest *request = [ServerRequest requestWithURL:url
                                                 requestMethod:RequestMethodGet
                                               responseHandler:^(id response, NSError *error) {
                                                   if([_delegate respondsToSelector:@selector(didRecieveResponse:withError:)]) {
                                                       [_delegate didRecieveResponse:response withError:error];
                                                   }
                                               }
                                  ];
        [request startConnection];
    }
}

- (void)updateObjectsWithParameters:(NSString *)parameters
{
    _objectsCount = 0;
    NSString *requestString = _APIMethod;
    if(parameters) {
        requestString = [requestString stringByAppendingFormat:@"?%@&count=%d&offset=0",parameters, _objectsInRequest];
    }
    else {
        requestString = [requestString stringByAppendingFormat:@"?count=%d&offset=0", _objectsInRequest];
    }
    requestString = [APIRequest signedRequest:requestString withParams:YES];
    NSURL *url = [NSURL URLWithString:requestString];
    ServerRequest *request = [ServerRequest requestWithURL:url
                                             requestMethod:RequestMethodGet
                                           responseHandler:^(id response, NSError *error) {
                                               [_loadedObjects removeAllObjects];
                                               if(!error) {
                                                   NSDictionary *responseDictionary = (NSDictionary *)response;
                                                   NSDictionary *pagingDictionary = [responseDictionary objectForKey:@"paging"];
                                                   _objectsCount = [[pagingDictionary objectForKey:@"fullcount"]integerValue];
                                               }
                                               if([_delegate respondsToSelector:@selector(didRecieveResponse:withError:)]) {
                                                   [_delegate didRecieveResponse:response withError:error];
                                               }
                                           }
                              ];
    [request startConnection];
}

- (BOOL)isNextObjectsPageAvailable
{
    return (self.loadedObjects.count < _objectsCount);
}

@end