//
//  URLRequest.h
//  TeKtoTut
//
//  Created by Admin on 01.12.12.
//
//

#import <Foundation/Foundation.h>

typedef enum {
    RequestMethodPost,
    RequestMethodGet,
    RequestMethodPut,
    RequestMethodDelete
}
RequestMethod;

typedef void (^RequestResponseHandler)(id response, NSError *error);

@interface URLRequest : NSObject <NSURLConnectionDataDelegate, NSURLConnectionDelegate>
{
    NSMutableURLRequest *   _request;
    NSURLConnection     *   _connection;
    NSMutableData       *   _downloadedData;
    RequestResponseHandler  _responseHandler;
}
@property (strong, nonatomic) NSString * requestID;

#pragma mark initializer methods
#pragma mark designated initializer

- (id)  initWithURL: (NSURL *) url
      requestMethod: (RequestMethod)requestMethod
    responseHandler: (RequestResponseHandler) responseHandler NS_RETURNS_RETAINED;

- (id)  initWithURLRequest: (NSURLRequest *) urlrequest
           responseHandler: (RequestResponseHandler) responseHandler NS_RETURNS_RETAINED;

#pragma mark class method-style initializers
+ (id) requestWithURL: (NSURL *) url
        requestMethod: (RequestMethod)requestMethod
      responseHandler: (RequestResponseHandler) responseHandler NS_RETURNS_NOT_RETAINED;

+ (id)  requestWithURLRequest: (NSURLRequest *) urlrequest
              responseHandler: (RequestResponseHandler) responseHandler NS_RETURNS_NOT_RETAINED;

#pragma mark defaults
/*
 These class methods determine the behavior of NSURLRequest and could be
 overridden in subclasses.
 */
+ (NSURLRequestCachePolicy) defaultCachePolicy;
+ (NSTimeInterval) defaultTimeoutInterval;

#pragma mark methods
// Connection methods
- (void) startConnection;
- (void) cancelConnection;
- (void) setRequestBody:(NSData *)data;

// Response handling methods
- (void) failWithError: (NSError *) error;
- (void) succeedWithResponse: (id) response;
- (void) preprocessResponse: (id) response;
@end
