//
//  URLRequest.m
//  TeKtoTut
//
//  Created by Admin on 01.12.12.
//
//

#import "URLRequest.h"

@implementation URLRequest

#pragma mark -
#pragma mark initialization methods

- (id) initWithURL: (NSURL *) url
      requestMethod: (RequestMethod)requestMethod
   responseHandler: (RequestResponseHandler) responseHandler
{
    self = [super init];
    
    if (self != nil)
    {
        _responseHandler = [responseHandler copy];
        
        _request = [[NSMutableURLRequest alloc]
                    initWithURL: url
                    cachePolicy: [[self class] defaultCachePolicy]
                    timeoutInterval: [[self class] defaultTimeoutInterval]];
        _request.HTTPMethod = [self httpMethodForRequestMethod:requestMethod];
    }
    return self;
}

- (id)initWithURLRequest:(NSURLRequest *)urlrequest
         responseHandler:(RequestResponseHandler)responseHandler
{
    self = [super init];
    if(self) {
        _responseHandler = [responseHandler copy];
        _request = [[NSMutableURLRequest alloc]initWithURL:urlrequest.URL cachePolicy:urlrequest.cachePolicy timeoutInterval:urlrequest.timeoutInterval];
        _request.HTTPMethod = urlrequest.HTTPMethod;
    }
    return self;
}
#pragma mark class method-style initializers
+ (id) requestWithURL: (NSURL *) url
        requestMethod: (RequestMethod)requestMethod
      responseHandler: (RequestResponseHandler) responseHandler
{
    return [[[self class] alloc] initWithURL: url
                               requestMethod:requestMethod
                             responseHandler: responseHandler];
}

+ (id)requestWithURLRequest:(NSURLRequest *)urlrequest responseHandler:(RequestResponseHandler)responseHandler
{
    return [[[self class]alloc]initWithURLRequest:urlrequest
                                  responseHandler:responseHandler];
}


#pragma mark dealloc
- (void) dealloc
{
    [self cancelConnectionAndCleanData];
}


#pragma mark -
#pragma mark Cleanup methods

- (void) cancelConnectionAndCleanData
{
    [_connection cancel];
    _connection     = nil;
    _downloadedData = nil;
}


#pragma mark -
#pragma mark Methods

- (NSString *)httpMethodForRequestMethod:(RequestMethod)requestMethod
{
    NSString *method = nil;
    switch (requestMethod) {
        case RequestMethodGet: {
            method = @"GET";
        }
            break;
        case RequestMethodPost: {
            method = @"POST";
        }
            break;
        case RequestMethodDelete: {
            method = @"DELETE";
        }
            break;
        case RequestMethodPut: {
            method = @"PUT";
        }
            break;
            
        default: {
            NSAssert(NO, @"Unknown http method");
        }
            break;
    }
    return method;
}

- (void) setRequestBody:(NSData *)data
{
    _request.HTTPBody = data;
}

- (void) startConnection
{
    dispatch_async(dispatch_get_main_queue(), ^(void){
        if (_connection == nil)
        {
            _connection = [[NSURLConnection alloc] initWithRequest: _request
                                                          delegate: self];
            _downloadedData = [NSMutableData data];
            [_connection start];
        }
    });
}

- (void) cancelConnection
{
    [self cancelConnectionAndCleanData];
}

- (void) failWithError: (NSError *) error
{
    [self cancelConnectionAndCleanData];
    
    if (_responseHandler != nil)
    {
        _responseHandler(nil, error);
    }
}

- (void) succeedWithResponse: (id) response
{
    [self cancelConnectionAndCleanData];
    
    if (_responseHandler != nil)
    {
        _responseHandler(response, nil);
    }
}

- (void) preprocessResponse: (id) response
{
    [self succeedWithResponse:response];
}

#pragma mark -
#pragma mark defaults
+ (NSURLRequestCachePolicy) defaultCachePolicy
{
    return NSURLRequestReloadIgnoringLocalCacheData;
}


+ (NSTimeInterval) defaultTimeoutInterval
{
    return 60.0;
}


#pragma mark -
#pragma mark NSURLConnectionDelegate
- (void)connection: (NSURLConnection *) connection
  didFailWithError: (NSError *)         error
{
    [self failWithError: error];
}


#pragma mark -
#pragma mark NSURLConnectionDataDelegate
- (void)connection: (NSURLConnection *) connection
didReceiveResponse: (NSURLResponse   *) response
{
    [_downloadedData setLength: 0];
}

- (void) connection: (NSURLConnection *) connection
     didReceiveData: (NSData *)          data
{
    [_downloadedData appendData: data];
}

- (void) connectionDidFinishLoading: (NSURLConnection *) connection
{
    [self preprocessResponse:_downloadedData];}
@end
