//
//  ScrollView.m
//  TeKtoTut
//
//  Created by mave on 2/18/13.
//
//

#import "ScrollPageView.h"

@implementation ScrollPageView
{
    NSInteger   _currentPage;
    CGPoint     _currentOffset;
}

#pragma mark - initialization
- (id)init
{
    self = [super init];
    if(self) {
        [self customInitialization];
    }
    return self;
}

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self customInitialization];
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if(self) {
        [self customInitialization];
    }
    return self;
}

- (void)customInitialization
{
    self.backgroundColor = [UIColor clearColor];
    self.delegate = self;
    UIActivityIndicatorView *actView = [[UIActivityIndicatorView alloc]initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
    [actView startAnimating];
    actView.center = CGPointMake(self.frame.origin.x + CGRectGetMidX(self.bounds), self.frame.origin.y + CGRectGetMidY(self.bounds));
    [self.superview addSubview:actView];
    
    [self reloadContent];
}

#pragma mark - working methods
- (void)reloadContent
{
    NSInteger numOfPages = [_dataSource numberOfPages];
    self.contentSize = CGSizeMake(numOfPages * 320, self.frame.size.height);
    self.contentOffset = CGPointMake(0, 0);
    _currentPage = 0;
    [_firstView removeFromSuperview];
    [_secondView removeFromSuperview];
    [_thirdView removeFromSuperview];
    _firstView = nil;
    _secondView = nil;
    _thirdView = nil;
    if(numOfPages) {
        _firstView = [_dataSource viewForIndex:0];
        _firstView.frame = CGRectMake(0, 0, 320, self.frame.size.height);
        [self addSubview:_firstView];
        if(numOfPages > 1) {
            _secondView = [_dataSource viewForIndex:1];
            _secondView.frame = CGRectMake(320, 0, 320, self.frame.size.height);
            [self addSubview:_secondView];
            if(numOfPages > 2) {
                _thirdView = [_dataSource viewForIndex:2];
                _thirdView.frame = CGRectMake(640, 0, 320, self.frame.size.height);
                [self addSubview:_thirdView];
            }
        }
    }
}

- (void)setPage:(NSInteger)page
{
    NSInteger numOfPages = [_dataSource numberOfPages];
    if(page < numOfPages) {
        [_firstView removeFromSuperview];
        [_secondView removeFromSuperview];
        [_thirdView removeFromSuperview];
        _firstView = nil;
        _secondView = nil;
        _thirdView = nil;
        _currentPage = page;
        NSInteger index;
        if(page == 0) {
            index = page + 1;
        }
        else if(page == (numOfPages - 1)) {
            index = page - 1;
        }
        else {
            index = page;
        }
        _firstView = [_dataSource viewForIndex:index - 1];
        _firstView.frame = CGRectMake((index - 1) * 320, 0, 320, self.frame.size.height);
        [self addSubview:_firstView];
        if(numOfPages > 1) {
            _secondView = [_dataSource viewForIndex:index];
            _secondView.frame = CGRectMake(index * 320, 0, 320, self.frame.size.height);
            [self addSubview:_secondView];
            if(numOfPages > 2) {
                _thirdView = [_dataSource viewForIndex:index + 1];
                _thirdView.frame = CGRectMake((index + 1) * 320, 0, 320, self.frame.size.height);
                [self addSubview:_thirdView];
            }
        }
        _currentOffset = CGPointMake(320 * _currentPage, 0);
        [self setContentOffset:_currentOffset animated:NO];
    }
}

#pragma mark - UIScrollView Delegate
- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
    NSInteger pageNum = (NSInteger)(self.contentOffset.x / 320.0);
    [self setPage:pageNum];
/*    if(fabs(self.contentOffset.x - _currentOffset.x) > 1E-5) {
        if(self.contentOffset.x < _currentOffset.x) {
            [self scrollToLeft];
        }
        else {
            [self scrollToRight];
        }
    }
 */
}

#pragma mark - Scroll Logic
- (void)scrollToLeft
{
    NSInteger numOfPages = [_dataSource numberOfPages];
    if(_currentPage != 1 && _currentPage != (numOfPages - 1)) {
        [_firstView removeFromSuperview];
        [_secondView removeFromSuperview];
        [_thirdView removeFromSuperview];
        _firstView = nil;
        _secondView = nil;
        _thirdView = nil;
        _firstView = [_dataSource viewForIndex:_currentPage - 2];
        _firstView.frame = CGRectMake((_currentPage - 2) * 320, 0, 320, self.frame.size.height);
        _secondView = [_dataSource viewForIndex:_currentPage - 1];
        _secondView.frame = CGRectMake((_currentPage - 1)* 320, 0, 320, self.frame.size.height);
        _thirdView = [_dataSource viewForIndex:_currentPage];
        _thirdView.frame = CGRectMake(_currentPage * 320, 0, 320, self.frame.size.height);
        
        [self addSubview:_firstView];
        [self addSubview:_secondView];
        [self addSubview:_thirdView];
    }
    _currentPage --;
    _currentOffset = CGPointMake(320 * _currentPage, 0);
    [self setContentOffset:_currentOffset animated:NO];
}

- (void)scrollToRight
{
    NSInteger numOfPages = [_dataSource numberOfPages];
    if(_currentPage != 1 && _currentPage != (numOfPages - 2)) {
        [_firstView removeFromSuperview];
        [_secondView removeFromSuperview];
        [_thirdView removeFromSuperview];
        _firstView = nil;
        _secondView = nil;
        _thirdView = nil;
        _firstView = [_dataSource viewForIndex:_currentPage];
        _firstView.frame = CGRectMake(_currentPage * 320, 0, 320, self.frame.size.height);
        _secondView = [_dataSource viewForIndex:_currentPage + 1];
        _secondView.frame = CGRectMake((_currentPage + 1)* 320, 0, 320, self.frame.size.height);
        _thirdView = [_dataSource viewForIndex:_currentPage + 2];
        _thirdView.frame = CGRectMake((_currentPage + 2)* 320, 0, 320, self.frame.size.height);
        
        [self addSubview:_firstView];
        [self addSubview:_secondView];
        [self addSubview:_thirdView];
    }
    _currentPage ++;
    _currentOffset = CGPointMake(320 * _currentPage, 0);
    [self setContentOffset:_currentOffset animated:NO];
}
@end
