//
//  CommentingTableViewCell.m
//  OpenOcean
//
//  Created by Alexander Chernyshev on 21/08/2012.
//  Copyright (c) 2012 Sibers. All rights reserved.
//

#import "TextInputView.h"

#import <QuartzCore/QuartzCore.h>

@implementation TextInputView
{
    CGRect _prevViewRect;
}
@synthesize delegate = _delegate;

#pragma mark - initialization method
- (void) awakeFromNib
{
    _textView.font = [UIFont systemFontOfSize:15.0];
    _textView.contentInset = UIEdgeInsetsMake(-9, 0.0, 0.0, 0.0);
    
    UIImage *image = [[UIImage imageNamed:@"text_field.png"] resizableImageWithCapInsets:UIEdgeInsetsMake(15.0, 20.0, 15.0, 20.0)];
    _backgroundTextImageView.image = image;
    
    [self addNotificationObserver];
}

- (void)dealloc
{
    [self removeNotificationObserver];
}

#pragma mark - Notifications
- (void)addNotificationObserver
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillHide:) name:UIKeyboardWillHideNotification object:nil];
}

- (void)removeNotificationObserver
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];
}

- (void)keyboardWillShow:(NSNotification *)notification
{
    CGRect endFrame = [[notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue];
    endFrame = [self.window convertRect:endFrame toView:self.superview];
    float duration = [[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey] floatValue];
    _prevViewRect = self.frame;
    [UIView animateWithDuration:duration animations:^{
        self.frame = CGRectMake(_prevViewRect.origin.x, endFrame.origin.y - _prevViewRect.size.height, _prevViewRect.size.width, _prevViewRect.size.height);
    }];
}

- (void)keyboardWillHide:(NSNotification *)notification
{
    float duration = [[notification.userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey] floatValue];
    [UIView animateWithDuration:duration animations:^{
        self.frame = _prevViewRect;
    }];
}

#pragma mark - initialization
- (void)customizeView
{
    //self.frame = CGRectMake(0, [UIScreen mainScreen].bounds.size.height - 40, 320, 40);
    _containerView = [[UIView alloc]initWithFrame:self.bounds];
    [self addSubview:_containerView];
    
    /*_optionsButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    _optionsButton .frame = CGRectMake(2, 3, 26, 18);
    [_optionsButton addTarget:self action:@selector(optionsButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [_containerView addSubview:_optionsButton];*/
    
    
    _sendButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    _sendButton.frame = CGRectMake(257, 3, 60, 21);
    _sendButton.titleLabel.font = [UIFont boldSystemFontOfSize:12.0];
    [_sendButton setTitle:NSLocalizedString(@"Послать", @"Послать") forState:UIControlStateNormal];
    [_sendButton addTarget:self action:@selector(sendButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [_containerView addSubview:_sendButton];
    
    _backgroundTextImageView = [[UIImageView alloc]initWithFrame:CGRectMake(30, 3, 224, 20)];
    UIImage *image = [[UIImage imageNamed:@"text_field.png"] resizableImageWithCapInsets:UIEdgeInsetsMake(15.0, 20.0, 15.0, 18.0)];
    _backgroundTextImageView.image = image;
    [_containerView addSubview:_backgroundTextImageView];
    
    _textView = [[UITextView alloc]initWithFrame:CGRectMake(30, 3, 217, 18)];
    _textView.delegate = self;
    _textView.scrollEnabled = NO;
    _textView.backgroundColor = [UIColor clearColor];
    [_containerView addSubview:_textView];
    
    _placeholderLabel = [[UILabel alloc]initWithFrame:CGRectMake(43, 3, 130, 18)];
    _placeholderLabel.text = NSLocalizedString(@"Сообщение", @"Сообщение");
    _placeholderLabel.backgroundColor = [UIColor clearColor];
    _placeholderLabel.textColor = [UIColor lightGrayColor];
    _placeholderLabel.font = [UIFont systemFontOfSize:14.0];
    [_containerView addSubview:_placeholderLabel];
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if(self) {
        [self customizeView];
    }
    return self;
}

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if(self) {
        [self customizeView];
    }
    return self;
}

#pragma mark - Actions
- (IBAction)sendButtonAction:(id)sender {
    if(_textView.text && ![_textView.text isEqualToString:@""]) {
        if([_delegate respondsToSelector:@selector(sendButtonActionWithText:)]) {
            [_delegate sendButtonActionWithText:_textView.text];
        }
        [_textView resignFirstResponder];
        _textView.text = nil;
        [self layoutCommentView];
        [self layoutCommentBackground];
        [self layoutView];
    }
}

/*- (IBAction)optionsButtonAction:(id)sender
{
    NSString *title = NSLocalizedString(@"Опции", @"Опции");
    NSString *winkTitle = NSLocalizedString(@"Подмигнуть", @"Подмигнуть");
    NSString *cancelTitle = NSLocalizedString(@"Отмена", @"Отмена");
    UIActionSheet *sheet = [[UIActionSheet alloc]initWithTitle:title
                                                      delegate:self
                                              cancelButtonTitle:cancelTitle
                                        destructiveButtonTitle:nil
                                             otherButtonTitles:winkTitle, nil];
    sheet.actionSheetStyle = UIActionSheetStyleBlackOpaque;
    [sheet showInView:self];
}*/

#pragma mark - layout methods
- (void)layoutCommentView
{
    NSString *text = _textView.text;
    if(!text || [text isEqualToString:@""]) {
        text = @"A";
    }
    else {
        const char *cString = [text UTF8String];
        char lastSimbol = cString[text.length - 1];
        if(lastSimbol == '\n') {
            text = [text stringByAppendingString:@"A"];
        }
    }
    CGSize size = [text sizeWithFont:_textView.font constrainedToSize:CGSizeMake(_textView.bounds.size.width - 16, 1000)];
    float newHeight = _containerView.frame.origin.y + _textView.frame.origin.y + size.height;
    if(newHeight <= 100) {
        _textView.scrollEnabled = NO;
        CGRect rect = _textView.frame;
        _textView.frame = CGRectMake(rect.origin.x, rect.origin.y, rect.size.width, size.height);
    }
    else {
        _textView.scrollEnabled = YES;
    }
}

- (void)layoutCommentBackground
{
    _backgroundTextImageView.frame = CGRectMake(_textView.frame.origin.x, _textView.frame.origin.y,
                                                224, _textView.frame.size.height);
}

- (void)layoutView
{
    CGRect oldFrame = self.frame;
    float oldHeight = oldFrame.size.height;
    
    float contViewHeight = _textView.frame.origin.y + _textView.frame.size.height + 6;
    float newHeight = _containerView.frame.origin.y + contViewHeight;
    
    float offset = newHeight - oldHeight;
    CGRect newFrame = CGRectMake(oldFrame.origin.x, oldFrame.origin.y - offset, oldFrame.size.width, newHeight);
    CGRect contViewFrame = CGRectMake(_containerView.frame.origin.x, _containerView.frame.origin.y, _containerView.frame.size.width, contViewHeight);
    
    [UIView animateWithDuration:0.25
                     animations:^(void) {
                         self.frame = newFrame;
                     }
                     completion:^(BOOL finished) {
                         _containerView.frame = contViewFrame;
                     }
     ];
}

#pragma mark - UITextViewDelegate method
- (void)textViewDidBeginEditing:(UITextView *)textView
{    
    if([textView.text isEqualToString:@""] || !textView.text) {
        if(![_containerView.subviews containsObject:_placeholderLabel]) {
            [_containerView addSubview:_placeholderLabel];
        }
        _sendButton.enabled = NO;
    }
    else {
        [_placeholderLabel removeFromSuperview];
        _sendButton.enabled = YES;
    }
}

- (void)textViewDidChange:(UITextView *)textView
{
    if([textView.text isEqualToString:@""] || !textView.text) {
        if(![_containerView.subviews containsObject:_placeholderLabel]) {
            [_containerView addSubview:_placeholderLabel];
        }
        _sendButton.enabled = NO;
    }
    else {
        [_placeholderLabel removeFromSuperview];
        _sendButton.enabled = YES;
    }
    
    [self layoutCommentView];
    [self layoutCommentBackground];
    [self layoutView];
}

/*#pragma mark - UIActionSheetDelegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    switch (buttonIndex) {
        case 0: {
            if([_delegate respondsToSelector:@selector(sendWink)]) {
                [_delegate sendWink];
            }
        }
            break;
        case 1: {
            //cancel action
        }
            break;
            
        default: {
            NSAssert(NO, @"Unknown button index");
        }
            break;
    }
}
*/
#pragma mark - working methods
- (BOOL)resignFirstResponder
{
    _textView.text = nil;
    [self textViewDidChange:_textView];
    return [_textView resignFirstResponder];
}

- (BOOL)isFirstResponder
{
    return [_textView isFirstResponder];
}
@end
